/*** 
 * \file gre/3dtextur.h
 * \brief Base class for texturing algorithm
 *
 * \if NODOC
 * $Id: 3dtextur.h_v 1.31 2012/06/07 21:01:31 mju Exp $
 *
 * $Log: 3dtextur.h_v $
 * Revision 1.31  2012/06/07 21:01:31  mju
 * Use const in drawBoundaryWall.
 *
 * Revision 1.30  2011/10/20 12:41:43  mju
 * Remove redundant hdr.
 *
 * Revision 1.29  2006/09/14 19:44:06  mju
 * Include region2d hdr.
 *
 * Revision 1.23  2004/09/20 17:37:00  vdronov
 * added fence drawing
 *
 * Revision 1.21  2004/08/12 22:17:23  vdronov
 * added SetPedestalcoordRef
 *
 * Revision 1.20  2004/08/12 19:39:10  vdronov
 * added ClearPedestal
 *
 * Revision 1.15  2004/06/17 17:26:47  vdronov
 * added formula texture class
 *
 * Revision 1.1  2003/03/10 15:08:04  vdronov
 * Initial revision
 * \endif
**/ 

#ifndef	INC_GRE_3DTEXTUR_H
#define	INC_GRE_3DTEXTUR_H

#ifndef	INC_GRE_3DUTILS_H
#include <gre/3dutils.h>
#endif

#ifndef	INC_GRE_3DSCENE_H
#include <gre/3dscene.h>
#endif

#ifndef	INC_MI32_REGION2D_H
#include <mi32/region2d.h>
#endif

#ifndef	INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif

namespace GRE {

#ifndef GENERATING_DOXYGEN_OUTPUT
class TEXTUREFILTER;
class CONTROLLER;
class TERRAIN;
class DRAWINGCONTEXT3D;
class BOUNDPARM;
#endif // GENERATING_DOXYGEN_OUTPUT

//! Base class for 3D texture models.
class TEXTURE {
	public:

	class IMPL_FACTORY {
		public:
			virtual TEXTURE *v_ConstructImpl () = 0;
		};

#define	DECLARE_TEXTURE_IMPL_FACTORY(name,model)							\
	class TEXTURE_IMPL_FACTORY_##name : public GRE::TEXTURE::IMPL_FACTORY {	\
		public:																\
		TEXTURE_IMPL_FACTORY_##name (										\
		) { GRE::TEXTURE::Register(this,model); }							\
		virtual GRE::TEXTURE *v_ConstructImpl () {							\
		return (new GRE::name);												\
			}																\
		};																	\
	static TEXTURE_IMPL_FACTORY_##name s_TextureImplFactory_##name;

		enum MODEL {
			MODEL_Default = 0,
			MODEL_SimpleRaster = 0,
			MODEL_Formula = 1,
			MODEL_Count = 2
			};

		static void Register (
			IMPL_FACTORY* factory,
			const MODEL model
			);

		//! GetName returns a name of the given model enum
		//!
		//! @return char pointer of the name
		static const char* GetName (
			MODEL model
			);

		//! GetDescription returns a description of the given model enum
		//!
		//! @return char pointer of the description
		static const char* GetDescription (
			MODEL model
			);

		//! GetInstance returns a new instance of the given model enum
		//!
		//! @return pointer to new model
		static TEXTURE* GetInstance (
			MODEL model
			);

		virtual ~TEXTURE (								//! Destructor
			);

		//! GetName returns a name of the model
		//!
		//! @return char pointer of the name
		const char* GetName (
			) const;

		//! GetDescription returns a description of the model
		//!
		//! @return char pointer of the description
		const char* GetDescription (
			) const;

		//! GetModel returns an enum of the model
		//!
		//! @return enum of the model
		const MODEL GetModel (
			) const {return m_Model; };

		//! Create new instance of texture model
		//!
		//! @return instance of texture model
		TEXTURE* CreateInstance (
			) { return v_CreateInstance(); };

		//!	Build internal structures for texture construction for given drape layer 
		//!
		//! @return 0 if built or < 0 if an error
		ERRVALUE Build (
			const GRE::LAYER* layer
			) { return v_Build(layer); };

		//!	Free internal structures for texture construction 
		void Free (
			) { v_Free(); return; };

		//!	Is internal structures for texture construction built? 
		//!
		//! @return true if built or false if an error
		bool IsBuilt (
			);

		//! Begining of texture retriving for given scene
		void Begin (
			const SCENE3D& scene
			) { v_Begin(scene); return; };

		//! Rnding of texture retriving
		void End (
			) { v_End(); return; };

		//! Get extents in map projection coordinates
		//!
		//! @return extents
		const DRECT2D& GetExtents (
			) { return v_GetExtents(); };

		//! Set model map projection be used for output data
		void SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			) { SetPedestalCoordRefSys(CoordRefSys); v_SetCoordRefSys(CoordRefSys); return; };

		//! Get map projection be used for output data 
		const SPATREF::COORDREFSYS GetCoordRefSys (
			) { return v_GetCoordRefSys(); };

		//! Get color for given cell and depth
		//!
		//! @return true if color was retrived
		bool GetColor (
			const INT32 line,
			const INT32 column,
			const UINT8 depth,
			COLOR& color 
			) { return v_GetColor(line, column, depth, color); };

		//! Get color for given point in specifyed map projection and depth
		//!
		//! @return true if color was retrived
		bool GetColor (
			const DPOINT2D& point,
			const UINT8 depth,
			COLOR& color 
			) { return v_GetColor(point, depth, color); };

		//! Check if for given depth pyramid tier exist
		//!
		//! @return the nearest lower depth
		UINT8 CheckDepth (
			const UINT8 depth
			) { return v_CheckDepth(depth); };

		//! Get transformation from object coordinates to specifyed map projection
		//!
		//! @return TRANS2D_MAPGEN&
		const TRANS2D_MAPGEN& GetTransformationInternalToModel (
			) { return v_GetTransformationInternalToModel(); };

		//!	Set boundary wall pedestal for terrain and texture
		//!
		//! @return >= 0 if set is done
		ERRVALUE SetBoundaryWall (
			const SCENE3D& scene,
			TERRAIN* terrain
			);

		//!	Draw given pedestal
		//!
		//! @return >= 0 if set is done
		ERRVALUE DrawBoundaryWall (
			CONTROLLER &controller,			 
			DRAWINGCONTEXT3D& drawing,
			const GRE::BOUNDPARM& boundparm
			);

		const REGION2D& GetBoundary (
			) { 
			if (m_Boundary.IsEmpty()) v_CreateBoundary(m_Boundary); 
			return m_Boundary;
			};


		void ClearPedestal (
			) { 
			m_PolyLineList.Clear();
			return;
			};


		void ClearBoundary (
			) { 
			m_Boundary.Clear();
			return;
			};

		bool IsBoundaryChanged (
			) const { 
			return m_Boundary.IsEmpty();
			};

	protected:

		TEXTURE (	
			const char* name,
			const char* desc,
			const MODEL model
			);

		//!	Set built state? 
		void SetBuilt (
			const bool built
			);

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef SIMPLE_ARRAY<INT32> BASEINDEX;
		typedef MILIST<BASEINDEX> BASEINDEXLIST;

		static IMPL_FACTORY* s_ImplFactories[MODEL_Count];

		const char* m_Name;
		const char* m_Description;
		const MODEL m_Model;

		bool m_Built;										//!<	Is an object built?

		REGION2D m_Boundary;

		REGION2D m_FinalBoundary;
		POLYLINELIST m_PolyLineList;
		BASEINDEXLIST m_BaseIndexList;
		SPATREF::COORDREFSYS m_PolyLineCoordRefSys;
		TRIANGULATION m_Triangulation;
		SCENE3D *m_Scene;

		TEXTURE (	
			);

		TEXTURE (											//! Copy constructor (not implemented)
			const TEXTURE &rhs
			);

		TEXTURE& operator= (								//! Assignment operator (not implemented)
			const TEXTURE& rhs
			);

		#endif // GENERATING_DOXYGEN_OUTPUT

		void SetPedestalCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			);

		virtual TEXTURE* v_CreateInstance (
			) = 0;

		virtual ERRVALUE v_Build (
			const GRE::LAYER *layer
			) = 0;

		virtual void v_Free (
			) = 0;

		virtual void v_Begin (
			const SCENE3D& scene
			) = 0;

		virtual void v_End (
			) = 0;

		virtual const DRECT2D& v_GetExtents (
			) = 0;

		virtual void v_CreateBoundary (
			REGION2D& boundary
			) = 0;

		virtual void v_SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			) = 0;

		virtual const SPATREF::COORDREFSYS v_GetCoordRefSys (
			) = 0;

		virtual bool v_GetColor (
			const INT32 line,
			const INT32 column,
			const UINT8 depth,
			COLOR& color 
			) = 0;

		virtual bool v_GetColor (
			const DPOINT2D& point,
			const UINT8 depth,
			COLOR& color 
			) = 0;

		virtual UINT8 v_CheckDepth (
			const UINT8 depth
			) = 0;

		virtual const TRANS2D_MAPGEN& v_GetTransformationInternalToModel (
			) = 0;

	};

}		// End of GRE namespace

#endif



