/**
 * \file mgui/edit.h
 * \brief Definitions for MGUI::CTRL_EDIT... and MGUI::FORM_EDIT... classes.
 *
 * \if NODOC
 * $Id: edit.h_v 1.53 2013/01/24 22:31:20 mju Exp $
 *
 * $Log: edit.h_v $
 * Revision 1.53  2013/01/24 22:31:20  mju
 * Remove menu stuff as should use menu_edit control instead.
 *
 * Revision 1.52  2013/01/18 22:40:10  mju
 * Add edit_text menu for typical edit operations.
 * Remove some legacy unimplemented methods.
 * Adjust virtual methods to allow override of onChangeSelection.
 *
 * Revision 1.51  2012/01/27 18:43:24  mju
 * Remove mfc stuff.
 *
 * Revision 1.50  2011/11/22 21:29:37  mju
 * Fix dox dependency.
 *
 * Revision 1.49  2011/09/28 21:25:55  mju
 * Add form_edit_number_base and use to derive form_edit_number/2.
 *
 * Revision 1.47  2011/02/18 15:31:04  mju
 * Add setRange variants that take separate minimum/maximum.
 *
 * Revision 1.46  2010/06/10 17:15:46  mju
 * Add edit_range::setUnitLabel.
 *
 * Revision 1.45  2010/01/27 15:06:31  mju
 * Allow -1 for decimal places to indicate automatic, change default to -2 for setFormat.
 *
 * Revision 1.44  2009/05/18 15:29:45  mju
 * Add dmsplaces member.
 *
 * Revision 1.43  2009/05/18 13:59:27  mju
 * Change degToStrFlags to dmstype so can set number of decimal places.
 *
 * Revision 1.42  2008/09/25 15:09:49  mju
 * Use createflags enum values for common flag values.
 *
 * Revision 1.41  2008/09/25 14:19:16  mju
 * Add flag_SizeInPixels for ctrl_edit_text.
 *
 * Revision 1.39  2008/01/14 21:23:14  mju
 * Add edit_text::getCaretPos.
 *
 * Revision 1.38  2008/01/14 16:32:15  mju
 * Add ctrl_text::handleKey.
 *
 * Revision 1.35  2007/01/31 17:00:35  dwilliss
 * Added flag to ctrl edit text to disable word-wrapping
 *
 * Revision 1.29  2006/12/15 21:11:28  dwilliss
 * Added Find() and ReplaceSelection() methods
 *
 * Revision 1.23  2006/10/06 16:36:39  mju
 * Use int for traversalDirection so don't need Xm defns.
 *
 * Revision 1.22  2006/10/04 22:52:09  dwilliss
 * moved syntax highlighting delegate to ctrl_edit_text (meaningless on single
 * line edit things)
 *
 * Revision 1.14  2006/02/28 18:51:14  mju
 * Add v_setFocus for forms.
 *
 * Revision 1.11  2006/02/01 16:04:58  dwilliss
 * Added support for fixed-width fonts
 *
 * Revision 1.10  2006/02/01 15:54:47  dwilliss
 * added flag for fixed width font
 *
 * Revision 1.1  2005/06/23 22:00:11  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_EDIT_H
#define	INC_MGUI_EDIT_H

#ifndef	INC_MGUI_EDITBASE_H
#include <mgui/editbase.h>
#endif

#ifndef	INC_MGUI_KEYCODE_H
#include <mgui/keycode.h>
#endif

#ifndef	INC_MI32_RANGE_H
#include <mi32/range.h>
#endif

#ifndef	INC_MI32_LATLONFM_H
#include <mi32/latlonfm.h>
#endif

#ifndef	INC_MI32_UNITCONV_H
#include <mi32/unitconv.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
struct COLOR;
struct XeTextSyntaxParseCallbackData;
#endif


namespace MGUI {

//===================================================================================================================

//! String 'edit' control.
class CTRL_EDIT_STRING : public MGUI::CTRL_EDIT_BASE {
	public:

		enum FLAGS {
			FLAG_Default =					CTRL_EDIT_BASE::CREATEFLAG_None,
			FLAG_RightJustify =			CTRL_EDIT_BASE::CREATEFLAG_RightAlign,				//!< Right-justify value, default is to left-justify
			FLAG_ReadOnly =				CTRL_EDIT_BASE::CREATEFLAG_ReadOnly,				//!< User cannot change value
			FLAG_Opaque =					CTRL_EDIT_BASE::CREATEFLAG_Opaque,					//!< Show '*'s for password input
			FLAG_WidthInPixels =			CTRL_EDIT_BASE::CREATEFLAG_WidthInPixels,			//!< Width specified in pixels
			FLAG_UseFixedWidthFont =	CTRL_EDIT_BASE::CREATEFLAG_UseFixedWidthFont,	//!< Use a fixed-width font if possible
			};

		//! Constructor.
		CTRL_EDIT_STRING ();

		//! Destructor.
		virtual ~CTRL_EDIT_STRING ();

		//! Clear to empty string.
		void ClearValue (
			bool notify = true							//!< Call OnChangeValue() if control has already been created
			);

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int maxlen,										//!< Maximum number of characters allowed in string, not including termination
			int width = 0,									//!< Width in 'typical' characters, minimum if can expand based on sizealign
			FLAGS flags = FLAG_Default,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get current string value.
		const MISTRING& GetValue ();

		//! Read value from INI file.
		//! If Create() has already been performed the control's state will be updated.
		void IniRead (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField,					//!< INI field to read from
			bool notify = true						//!< Call OnChangeValue() if control has already been created
			);

		//! Write current value to INI file.
		void IniWrite (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField						//!< INI field to read from
			) const { ::IniWrite(IniHandle,IniGroup,IniField,m_value); }

		//! Set the current cursor (a.k.a. caret) location.
		void SetCursorPos (
			int nCharIndex
			);

		//! Set delegate to call to perform validation.
		void SetDelegateOnValidate (
			FastDelegate<bool(MISTRING&)> delegate
			) { m_DelegateOnValidate = delegate; }

		//! Set current string value with validation.
		//! OnValidate() will be called and if valid, the string will be updated.
		//! No comparison is made with the current string value.
		void SetValue (
			const MIUNICODE* string,					//!< New string to set
			bool notify = true						//!< Call OnChangeValue() if string actually updated
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		FastDelegate<bool(MISTRING&)> m_DelegateOnValidate;
		MISTRING m_value;

		bool OnValidate (MISTRING& string) const;

		virtual void v_UpdateValue (MISTRING& newvalue);

		CTRL_EDIT_STRING (const CTRL_EDIT_STRING&);
		CTRL_EDIT_STRING& operator= (const CTRL_EDIT_STRING&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Validate the string and alter if possible/necessary.
		//!	@return 'true' if string could be validated, 'false' if not
		virtual bool v_OnValidate (
			MISTRING& string							//!< String to be validated
			) const;

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::CTRL_EDIT_STRING::FLAGS)
#endif

//===================================================================================================================

//! Form consisting of label and string edit field.
//!	Use GetEditCtrl() to access the CTRL_EDIT_STRING control and set delegates, etc on it.
class FORM_EDIT_STRING : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_EDIT_STRING ();

		//! Destructor.
		virtual ~FORM_EDIT_STRING ();

		//! Clear to empty string.
		void ClearValue (
			bool notify = true							//!< Call OnChangeValue() if control has already been created
			) { m_editctrl.ClearValue(notify); }

		//! Create form with Unicode label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label
			int maxlen,										//!< Maximum number of characters allowed in string, not including termination
			int width = 0,									//!< Width in 'typical' characters, minimum if can expand based on sizing
			MGUI::CTRL_EDIT_STRING::FLAGS flags = MGUI::CTRL_EDIT_STRING::FLAG_Default,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight,
			MGUI::CTRL_LABEL::STYLE labelstyle = MGUI::CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Retrieve string "edit" control for form.
		MGUI::CTRL_EDIT_STRING& GetEditCtrl (
			) { return (m_editctrl); }

		//! Retrieve label control for form.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel (
			) { return (m_label); }

		//! Get current string value.
		const MISTRING& GetValue (
			) { return (m_editctrl.GetValue()); }

		//! Determine if the edit form control has a value
		bool HasValue (
			) const { return (m_editctrl.HasValue()); }

		//! Read value from INI file.
		//! If Create() has already been performed the control's state will be updated.
		void IniRead (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField,					//!< INI field to read from
			bool notify = true						//!< Call OnChangeValue() if control has already been created
			) { m_editctrl.IniRead(IniHandle,IniGroup,IniField,notify); }

		//! Write current value to INI file.
		void IniWrite (
			INIHANDLE IniHandle,						//!< Handle to INI file to write to, 0 for default
			const char *IniGroup,					//!< INI group to write to
			const char *IniField						//!< INI field to write to
			) const { m_editctrl.IniWrite(IniHandle,IniGroup,IniField); }

		//! Set delegate to call when user presses \<Enter\> in edit control.
		void SetDelegateOnActivate (
			DELEGATE_VOID_NOPARMS delegate
			) { m_editctrl.SetDelegateOnActivate(delegate); }

		//! Set delegate to call when value is changed after validation.
		//! If the user modifies the text in the edit control will not be called until the control loses "focus".
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			) { m_editctrl.SetDelegateOnChangeValue(delegate); }

		//! Set delegate to call when user modifies the text in the edit control.
		//! The actual value does not get updated until the control loses focus or GetValue() is called.
		void SetDelegateOnUserEdit (
			DELEGATE_VOID_NOPARMS delegate
			) { m_editctrl.SetDelegateOnUserEdit(delegate); }

		//! Set delegate to call to perform validation.
		void SetDelegateOnValidate (
			FastDelegate<bool(MISTRING&)> delegate
			) { m_editctrl.SetDelegateOnValidate(delegate); }

		//! Set label text from Unicode string.
		void SetLabel (
			const MISTRING& label						//!< New label text
			) { m_label.SetLabel(label); }

		//! Select range of characters in string.
		void SetSelection (
			int StartChar = 0,						//!< Starting character position in string, 0 for beginning
			int EndChar = -1,							//!< Ending character position, -1 for end of string
			bool NoScroll = true						//!< Indicates whether to scroll caret into view
			) { m_editctrl.SetSelection(StartChar,EndChar,NoScroll); }

		//! Set set of 'valid' or 'invalid' characters.
		//! This does not alter the current contents, if any.
		void SetValidChars (
			const MIUNICODE* string,					//!< Characters valid or invalid to enter
			bool invert = false						//!< Allow all but specified characters
			) { m_editctrl.SetValidChars(string,invert); }

		//! Set current string value with validation.
		//! OnValidate() will be called and if valid, the string will be updated.
		//! No comparison is made with the current string value.
		void SetValue (
			const MIUNICODE* string,					//!< New string to set
			bool notify = true						//!< Call OnChangeValue() if string actually updated
			) { m_editctrl.SetValue(string,notify); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_LABEL m_label;
		CTRL_EDIT_STRING m_editctrl;
		// MGUI::FORM overrides.
		virtual void v_SetFocus () const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Number edit control.
class CTRL_EDIT_NUMBER : public MGUI::CTRL_EDIT_BASE {
	public:

		enum FLAGS {
			FLAG_Default =					CTRL_EDIT_BASE::CREATEFLAG_None,
			FLAG_ReadOnly =				CTRL_EDIT_BASE::CREATEFLAG_ReadOnly,				//!< User cannot change value
			FLAG_Opaque =					CTRL_EDIT_BASE::CREATEFLAG_Opaque,					//!< Show '*'s for password input
			FLAG_WidthInPixels =			CTRL_EDIT_BASE::CREATEFLAG_WidthInPixels,			//!< Width specified in pixels
			FLAG_UseFixedWidthFont =	CTRL_EDIT_BASE::CREATEFLAG_UseFixedWidthFont,	//!< Use a fixed-width font if possible
			FLAG_NoMin =					0x0001,	//!< No minimum value enforced
			FLAG_NoMax =					0x0002,	//!< No maximum value enforced
			FLAG_NoRange =					0x0003,	//!< No range enforcement
			FLAG_NoDefault =				0x0004,	//!< No default specified in creation
			FLAG_AddOne =					0x0008,	//!< Add 1.0 to value shown to user
			FLAG_BlankZero =				0x0020,	//!< Blank the field if value is 0.0
			FLAG_LeftJustify =			0x0080	//!< Left-justify value, default is to right-justify
			};

		enum FORMAT {
			FORMAT_Decimal,
			FORMAT_Exponential,
			FORMAT_Latitude,
			FORMAT_Longitude,
			FORMAT_DegMinSec
			};

		//! Constructor.
		CTRL_EDIT_NUMBER ();

		//! Destructor.
		virtual ~CTRL_EDIT_NUMBER ();

		//! Clear the value.
		void ClearValue (
			bool notify = true							//!< Call OnChangeValue() if control has already been created
			);

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			double dft,										//!< Default value
			double min,										//!< Minimum value
			double max,										//!< Maximum value
			int places = 0,								//!< Number of decimal places to show, -1 for automatic
			int width = 0,									//!< Width in "characters", 0 to automatically determine based on min/max/places
			FLAGS flags = FLAG_Default,				//!< Flags
			FORMAT format = FORMAT_Decimal,			//!< Format
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get allowed numeric range.
		//!	@return Allowed range.
		const DOUBLE_RANGE& GetRange (
			) const { return (m_range); }

		//! Get current value.
		double GetValue ();

		//! Determine if has valid value.
		bool HasValue ();

		//! Read value from INI file.
		//! If Create() has already been performed the control's state will be updated.
		void IniRead (
			INIHANDLE IniHandle,							//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,						//!< INI group to read from
			const char *IniField,						//!< INI field to read from
			bool notify = true							//!< Call OnChangeValue() if control has already been created
			);

		//! Write current value to INI file.
		void IniWrite (
			INIHANDLE IniHandle,							//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,						//!< INI group to read from
			const char *IniField							//!< INI field to read from
			) const { ::IniWrite(IniHandle,IniGroup,IniField,m_value); }

		//! Set format for showing/entering values.
		//! Setting to Latitude, Longitude or DegMinSec will also adjust range.
		void SetFormat (
			FORMAT format,
			int places = -2,								//!< Decimal places, -1 for automatic, -2 to leave unchanged
			bool notify = true
			);
/*
		//! Set format for displaying latitude/longitude values.
		void SetLatLonFormat (
			DEGTOSTRFLAGS flags
			);
*/
		//! Set allowed range.
		//! @return 'true' if resulted in value being modified, 'false' if not.
		bool SetRange (
			const DOUBLE_RANGE& range,					//!< New range to set
			bool notify = true,							//!< Call OnChangeValue() if value adjusted to new range
			FLAGS flags = FLAG_Default					//!< FLAG_NoMin, FLAG_NoMax, FLAG_NoRange
			);

		//! Set allowed range.
		//! @return 'true' if resulted in value being modified, 'false' if not.
		bool SetRange (
			double minimum,								//!< New minimum value
			double maximum,								//!< New maximum value
			bool notify = true,							//!< Call OnChangeValue() if value adjusted to new range
			FLAGS flags = FLAG_Default					//!< FLAG_NoMin, FLAG_NoMax, FLAG_NoRange
			);

		//! Set current value.
		void SetValue (
			double value,
			bool notify = true
			);

		//! Set unit conversion for displayed value.
		void SetUnitConversion (
			const UNITCONV& unitconv
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		CTRL_LABEL m_UnitLabel;
		FORMAT m_Format;
		FLAGS m_flags;
		DMSTYPE m_DegToStrFlags;
		INT16 m_places;
		INT16 m_dmsplaces;
		double m_value;
		DOUBLE_RANGE m_range;
		UNITCONV m_unitconv;

		bool CheckRange (double& value) const;
//		void SetDegToStrFlags (DEGTOSTRFLAGS flags);
		void UpdateText ();

		virtual void v_UpdateValue (MISTRING& newvalue);

		CTRL_EDIT_NUMBER (const CTRL_EDIT_NUMBER&);
		CTRL_EDIT_NUMBER& operator= (const CTRL_EDIT_NUMBER&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::CTRL_EDIT_NUMBER::FLAGS)
#endif

//===================================================================================================================

//! Base for simple forms contain single numeric field.
//! Use GetEditCtrl() to access the CTRL_EDIT_NUMBER control and set delegates, etc on it.
class FORM_EDIT_NUMBER_BASE : public MGUI::FORM_COMPOSITE {
	public:
		//! Constructor.
		FORM_EDIT_NUMBER_BASE ();

		//! Destructor.
		virtual ~FORM_EDIT_NUMBER_BASE ();

		//! Clear the value.
		void ClearValue (
			bool notify = true							//!< Call OnChangeValue() if control has already been created
			) { m_editctrl.ClearValue(notify); }

		//! Retrieve string "edit" control for form.
		MGUI::CTRL_EDIT_NUMBER& GetEditCtrl (
			) { return (m_editctrl); }

		//! Get allowed numeric range.
		//!	@return Allowed range.
		const DOUBLE_RANGE& GetRange (
			) const { return (m_editctrl.GetRange()); }

		//! Get current value.
		double GetValue (
			) { return (m_editctrl.GetValue()); }

		//! Determine if has valid value.
		bool HasValue (
			) { return (m_editctrl.HasValue()); }

		//! Read value from INI file.
		//! If Create() has already been performed the control's state will be updated.
		void IniRead (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField,					//!< INI field to read from
			bool notify = true						//!< Call OnChangeValue() if control has already been created
			) { m_editctrl.IniRead(IniHandle,IniGroup,IniField,notify); }

		//! Write current value to INI file.
		void IniWrite (
			INIHANDLE IniHandle,						//!< Handle to INI file to write to, 0 for default
			const char *IniGroup,					//!< INI group to write to
			const char *IniField						//!< INI field to write to
			) const { m_editctrl.IniWrite(IniHandle,IniGroup,IniField); }

		//! Set delegate to call when user presses \<Enter\> in edit control.
		void SetDelegateOnActivate (
			DELEGATE_VOID_NOPARMS delegate
			) { m_editctrl.SetDelegateOnActivate(delegate); }

		//! Set delegate to call when value is changed after validation.
		//! If the user modifies the text in the edit control will not be called until the control loses "focus".
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			) { m_editctrl.SetDelegateOnChangeValue(delegate); }

		//! Set delegate to call when user modifies the text in the edit control.
		//! The actual value does not get updated until the control loses focus or GetValue() is called.
		void SetDelegateOnUserEdit (
			DELEGATE_VOID_NOPARMS delegate
			) { m_editctrl.SetDelegateOnUserEdit(delegate); }

		//! Set format for showing/entering values.
		//! Setting to Latitude, Longitude or DegMinSec will also adjust range.
		void SetFormat (
			MGUI::CTRL_EDIT_NUMBER::FORMAT format,
			int places = -1,							//!< Decimal places for FORMAT_Decimal or FORMAT_Exponential, -1 to leave unchanged
			bool notify = true
			) { m_editctrl.SetFormat(format,places,notify); }

		//! Set allowed range.
		//! @return 'true' if resulted in value being modified, 'false' if not.
		bool SetRange (
			const DOUBLE_RANGE& range,				//!< New range to set
			bool notify = true,						//!< Call OnChangeValue() if value adjusted to new range
			MGUI::CTRL_EDIT_NUMBER::FLAGS flags = MGUI::CTRL_EDIT_NUMBER::FLAG_Default		//!< FLAG_NoMin, FLAG_NoMax, FLAG_NoRange
			) { return (m_editctrl.SetRange(range,notify,flags)); }

		//! Set allowed range.
		//! @return 'true' if resulted in value being modified, 'false' if not.
		bool SetRange (
			double minimum,								//!< New minimum value
			double maximum,								//!< New maximum value
			bool notify = true,						//!< Call OnChangeValue() if value adjusted to new range
			MGUI::CTRL_EDIT_NUMBER::FLAGS flags = MGUI::CTRL_EDIT_NUMBER::FLAG_Default		//!< FLAG_NoMin, FLAG_NoMax, FLAG_NoRange
			) { return (m_editctrl.SetRange(minimum,maximum,notify,flags)); }

		//! Set whether value is read-only or not.
		void SetReadOnly (
			bool ReadOnly = true						//!< True if read only, false if not
			) { m_editctrl.SetReadOnly(ReadOnly); }

		//! Set current value.
		void SetValue (
			double value,								//!< New value to set
			bool notify = true						//!< Call OnChangeValue() if change occurred
			) { m_editctrl.SetValue(value,notify); }

		//! Set unit conversion for displayed value.
		void SetUnitConversion (
			const UNITCONV& unitconv				//!< Unit conversion definition
			) { m_editctrl.SetUnitConversion(unitconv); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_EDIT_NUMBER m_editctrl;
		// MGUI::FORM overrides.
		virtual void v_SetFocus () const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Form consisting of single label and numeric edit field.
class FORM_EDIT_NUMBER : public FORM_EDIT_NUMBER_BASE {
	public:

		//! Constructor.
		FORM_EDIT_NUMBER ();

		//! Destructor.
		virtual ~FORM_EDIT_NUMBER ();

		//! Create the control with label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			const MISTRING& label,					//!< Label
			double dft,									//!< Default value
			double min,									//!< Minimum value
			double max,									//!< Maximum value
			int places = 0,							//!< Number of decimal places to show
			int width = 0,								//!< Width in "characters", 0 to automatically determine based on min/max/places
			CTRL_EDIT_NUMBER::FLAGS flags = CTRL_EDIT_NUMBER::FLAG_Default,			//!< Flags
			CTRL_EDIT_NUMBER::FORMAT format = CTRL_EDIT_NUMBER::FORMAT_Decimal,		//!< Format
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize,
			CTRL_LABEL::STYLE labelstyle = CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Retrieve label control for form.
		//! Usually the label control is only retrieved for alignment purposes.
		CTRL_LABEL& GetLabel (
			) { return (m_label); }

		//! Set label text from Unicode string.
		void SetLabel (
			const MISTRING& label						//!< New label text
			) { m_label.SetLabel(label); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_LABEL m_label;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Form consisting of numeric edit field with labels before and after.
class FORM_EDIT_NUMBER2 : public FORM_EDIT_NUMBER_BASE {
	public:

		//! Constructor.
		FORM_EDIT_NUMBER2 ();

		//! Destructor.
		virtual ~FORM_EDIT_NUMBER2 ();

		//! Create the control with labels.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			const MISTRING& labelBefore,			//!< Text for label "before" edit control
			const MISTRING& labelAfter,			//!< Text for label "after" edit control
			double dft,									//!< Default value
			double min,									//!< Minimum value
			double max,									//!< Maximum value
			int places = 0,							//!< Number of decimal places to show
			int width = 0,								//!< Width in "characters", 0 to automatically determine based on min/max/places
			CTRL_EDIT_NUMBER::FLAGS flags = CTRL_EDIT_NUMBER::FLAG_Default,			//!< Flags
			CTRL_EDIT_NUMBER::FORMAT format = CTRL_EDIT_NUMBER::FORMAT_Decimal,		//!< Format
			LAYOUT_SIZEALIGN EditSizeAlign = LAYOUT_SIZEALIGN_FixedSize,
			LAYOUT_SIZEALIGN LabelAfterSizeAlign = LAYOUT_SIZEALIGN_FixedSize			//!< Sizing for label "after" edit control
			);

		//! Retrieve label control "after" edit control.
		//! Usually the label control is only retrieved for alignment purposes.
		CTRL_LABEL& GetLabelAfter (
			) { return (m_labelAfter); }

		//! Set text for label "after" edit control.
		void SetLabelAfter (
			const MISTRING& label						//!< New label text
			) { m_labelAfter.SetLabel(label); }

		//! Retrieve label control "before" edit control.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabelBefore (
			) { return (m_labelBefore); }

		//! Set text for label "before" edit control.
		void SetLabelBefore (
			const MISTRING& label						//!< New label text
			) { m_labelBefore.SetLabel(label); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_LABEL m_labelBefore;
		CTRL_LABEL m_labelAfter;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Form containing pair of numeric edit fields for specifying a range.
class FORM_EDIT_RANGE : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_EDIT_RANGE ();

		//! Destructor.
		virtual ~FORM_EDIT_RANGE ();

		//! Clear the value.
		void ClearValue (
			bool notify = true							//!< Call OnChangeValue() if form has already been created
			);

		//! Create form with no label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const DOUBLE_RANGE& dft,					//!< Default value
			double min,										//!< Minimum value
			double max,										//!< Maximum value
			double mininterval = 0,						//!< Minimum interval between values
			int places = 0,								//!< Number of decimal places to show
			int width = 0,									//!< Width in "characters", 0 to automatically determine based on min/max/places
			CTRL_EDIT_NUMBER::FLAGS flags = CTRL_EDIT_NUMBER::FLAG_Default,				//!< Flags
			CTRL_EDIT_NUMBER::FORMAT format = CTRL_EDIT_NUMBER::FORMAT_Decimal,			//!< Format
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Create the form with Unicode label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label
			const DOUBLE_RANGE& dft,					//!< Default value
			double min,										//!< Minimum value
			double max,										//!< Maximum value
			double mininterval = 0,						//!< Minimum interval between values
			int places = 0,								//!< Number of decimal places to show
			int width = 0,									//!< Width in "characters", 0 to automatically determine based on min/max/places
			CTRL_EDIT_NUMBER::FLAGS flags = CTRL_EDIT_NUMBER::FLAG_Default,				//!< Flags
			CTRL_EDIT_NUMBER::FORMAT format = CTRL_EDIT_NUMBER::FORMAT_Decimal,			//!< Format
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			MGUI::CTRL_LABEL::STYLE labelstyle = MGUI::CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Get edit control for maximum value.
		MGUI::CTRL_EDIT_NUMBER& GetEditCtrlMax (
			) { return (m_MaxEN); }

		//! Get edit control for minimum value.
		MGUI::CTRL_EDIT_NUMBER& GetEditCtrlMin (
			) { return (m_MinEN); }

		//! Get allowed numeric range.
		//!	@return Allowed range.
		const DOUBLE_RANGE& GetRange (
			) const { return (m_MinEN.GetRange()); }

		//! Retrieve label control for form.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel (
			) { return (m_RangeLabel); }

		//! Get current value.
		DOUBLE_RANGE GetValue (
			) { return (DOUBLE_RANGE(m_MinEN.GetValue(),m_MaxEN.GetValue())); }

		//! Determine if has valid value.
		bool HasValue (
			) { return (m_MinEN.HasValue() && m_MaxEN.HasValue()); }

		//! Set whether to automatically swap values if minimum > maximum.
		void SetAutoSwap (
			bool AutoSwap = true
			) { m_AutoSwap = AutoSwap; }

		//! Set delegate to call when either minimum or maximum value is changed after validation.
		//! If the user modifies the text in the edit control will not be called until the control loses "focus".
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeValue = delegate; }

		//! Set format for showing/entering values.
		//! Setting to Latitude, Longitude or DegMinSec will also adjust range.
		void SetFormat (
			CTRL_EDIT_NUMBER::FORMAT format,
			int places = -1							//!< Decimal places for FORMAT_Decimal or FORMAT_Exponential, -1 to leave unchanged
			) {
			m_MinEN.SetFormat(format,places);
			m_MaxEN.SetFormat(format,places);
			}

		//! Set label text from Unicode string.
		void SetLabel (
			const MISTRING& label						//!< New label text
			) { m_RangeLabel.SetLabel(label); }
/*
		//! Set format for displaying latitude/longitude values.
		void SetLatLonFormat (
			DEGTOSTRFLAGS flags
			) {
			m_MinEN.SetLatLonFormat(flags);
			m_MaxEN.SetLatLonFormat(flags);
			}
*/
		//! Set minimum allowed interval between minimum and maximum.
		//! @return 'true' if resulted in value being modified, 'false' if not.
		bool SetMinInterval (
			double MinInterval,
			bool notify = true
			);

		//! Set allowed range.
		//! @return 'true' if resulted in value being modified, 'false' if not.
		bool SetRange (
			const DOUBLE_RANGE& range,					//!< New range to set
			bool notify = true,							//!< Call OnChangeValue() if value adjusted to new range
			CTRL_EDIT_NUMBER::FLAGS flags = CTRL_EDIT_NUMBER::FLAG_Default				//!< FLAG_NoMin, FLAG_NoMax, FLAG_NoRange
			);

		//! Set allowed range.
		//! @return 'true' if resulted in value being modified, 'false' if not.
		bool SetRange (
			double minimum,								//!< New minimum value
			double maximum,								//!< New maximum value
			bool notify = true,							//!< Call OnChangeValue() if value adjusted to new range
			CTRL_EDIT_NUMBER::FLAGS flags = CTRL_EDIT_NUMBER::FLAG_Default				//!< FLAG_NoMin, FLAG_NoMax, FLAG_NoRange
			);

		//! Set whether value is read-only or not.
		void SetReadOnly (
			bool ReadOnly = true						//!< True if read only, false if not
			) {
			m_MinEN.SetReadOnly(ReadOnly);
			m_MaxEN.SetReadOnly(ReadOnly);
			}

		//! Set unit conversion for displayed value.
		void SetUnitConversion (
			const UNITCONV& unitconv
			) {
			m_MinEN.SetUnitConversion(unitconv);
			m_MaxEN.SetUnitConversion(unitconv);
			}

		//! Set unit label text.
		void SetUnitLabel (
			const MISTRING& label
			);

		//! Set current value.
		void SetValue (
			const DOUBLE_RANGE& value,
			bool notify = true
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DELEGATE_VOID_NOPARMS m_DelegateOnChangeValue;
		CTRL_LABEL m_RangeLabel;
		CTRL_EDIT_NUMBER m_MinEN;
		CTRL_LABEL m_ToLabel;
		CTRL_EDIT_NUMBER m_MaxEN;
		CTRL_LABEL m_UnitLabel;
		double m_MinInterval;
		bool m_AutoSwap;

		bool AdjustForInterval (CTRL_EDIT_NUMBER *pEN = 0);

		void CreateMinToMax (
			const DOUBLE_RANGE& dft,
			double min,
			double max,
			double mininterval,
			int places,
			int width,
			CTRL_EDIT_NUMBER::FLAGS flags,
			CTRL_EDIT_NUMBER::FORMAT format,
			MGUI::LAYOUT_SIZEALIGN sizealign
			);

		void NotifyChangeValue ();
		void OnChangeValueMax ();
		void OnChangeValueMin ();

		// MGUI::FORM overrides.
		virtual void v_SetFocus () const;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Multi-line text 'edit' control.
class CTRL_EDIT_TEXT : public MGUI::CTRL_EDIT_BASE {
	public:

		enum FLAGS {
			FLAG_Default =					CTRL_EDIT_BASE::CREATEFLAG_None,
			FLAG_RightJustify =			CTRL_EDIT_BASE::CREATEFLAG_RightAlign,				//!< Right-justify value, default is to left-justify
			FLAG_ReadOnly =				CTRL_EDIT_BASE::CREATEFLAG_ReadOnly,				//!< User cannot change value
			FLAG_Opaque =					CTRL_EDIT_BASE::CREATEFLAG_Opaque,					//!< Show '*'s for password input
			FLAG_WidthInPixels =			CTRL_EDIT_BASE::CREATEFLAG_WidthInPixels,			//!< Width specified in pixels
			FLAG_UseFixedWidthFont =	CTRL_EDIT_BASE::CREATEFLAG_UseFixedWidthFont,	//!< Use a fixed-width font if possible
			FLAG_SizeInPixels =			CTRL_EDIT_BASE::CREATEFLAG_SizeInPixels,			//!< Size specified in pixels
			FLAG_WYSIWYG =					0x0001,	//!< Do WYSIWYG editing.  Parses {~,,.} codes for text style
			FLAG_NoWordWrap =				0x0002,	//!< Give a horizontal scroll bar instead of word-wrapping
			};

		typedef fastdelegate::FastDelegate<void(INT32 lin, INT32 col)> DELEGATE_ONCURSORMOTION;

		//! Constructor.
		CTRL_EDIT_TEXT ();

		//! Destructor.
		virtual ~CTRL_EDIT_TEXT ();

		//! Determine if the last edit operation can be undone by a call to the Undo member function.
		bool CanUndo () const;

		//! Clear entire control to empty string.
		void ClearValue (
			bool notify = true						//!< Call OnChangeValue() if control has already been created
			);

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			int width = 0,								//!< Width in 'typical' characters, minimum if can expand based on sizealign
			int height = 0,							//!< Height lines, minimum if can expand based on sizealign
			FLAGS flags = FLAG_Default,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Searches the text in the control for a given string.
		//! If bNext is true, the search begins one character past the current 
		//! cursor location.
		//! If the string is found, it will be selected and the function will
		//! return true.  If not found, it will beep and return false.
		bool Find (
			const MISTRING& SearchString,
			bool bCaseSensitive = true,
			bool bNext = false
			);

		//! Get the caret position
		//! @return true if position valid, false if not.
		bool GetCaretPos (
			int& caretrow,
			int& caretcolumn
			) const;

		//! Determines the topmost visible line in an edit control.
		//! Returns the zero-based index of the topmost visible line. For single-line edit controls, 
		//!	the return value is 0.
		int GetFirstVisibleLine () const;

		//! Get the number of lines in the edit control.
		int GetLineCount () const;

		//! Retrieves the line number of the line that contains the specified character index.
		//! Returns the zero-based line number of the line containing the character index specified by nIndex. 
		//! If nIndex is -1, the number of the line that contains the first character of the selection 
		//! is returned. If there is no selection, the current line number is returned.
		int GetLineFromChar (
			int nIndex = -1
			) const;

		//! Retrieves the character index of a line within a multiple-line edit control.
		//! Returns the character index of the line specified in nLine or -1 if the specified line 
		//! number is greater then the number of lines in the edit control.
		int GetLineIndex (
			int nLine = -1
			);

		//! Get current string value.
		const MISTRING& GetValue ();

		//! Handle keypress.
		void HandleKey (
			MGUI::KEYCODE keycode,
			MGUI::KEYSTATE keystate
			);

		//! Hide the cursor (a.k.a. caret).
		void HideCursor ();

		//! Add edit items to menu and link to this control instance.
		void MenuAddEditItems (
			MGUI::MENU& menu			//!< Menu to add items to, must have been created first
			);

		//! Set the current cursor (a.k.a. caret) location.
		void SetCursorPos (
			int nCharIndex,
			bool bScroll = true	//!< True to scroll so cursor is visible in the window.
			);

		//! Set the current cursor (a.k.a. caret) location
		void SetCursorPos (
			int line,
			int col,
			bool bScroll = true	//!< True to scroll so cursor is visible in the window.
			);

		//! Set the delegate to call when the user clicks a hyper link in the text.
		//! Note: This is for the X version only and is really meant for use
		//!   on a ReadOnly control.
		//! A Hyper link can be embeded in text by using the following escape
		//! codes:
		//!   \<ESC\>[6m the text to show \<ESC\>[5m the href \<ESC\>[25m
		//! The text between the \<ESC\>[5m and \<ESC\>[25m will not be shown to
		//! the user, but will be passed to the delegate if the user clicks on
		//! it.  The only visual indication the user gets that it's a link is
		//! that the cursor changes to a hand when they hover over the spot.
		void SetDelegateOnHyperLink (
			DELEGATE_VOID_MISTRING delegate
			) { m_DelegateOnHyperLink = delegate; }

		//! Set a delegate to be called when the cursor is moved.
		//! The delegate will be called with the current line and column numbers
		//! (in that order and 0-based).
		void SetDelegateOnCursorMotion (
			DELEGATE_ONCURSORMOTION delegate
			) { m_DelegateOnCursorMotion = delegate; }

#ifndef GENERATING_DOXYGEN_OUTPUT
		// Note: this is for use by FORM_SMLEDITOR.  It uses some ugly 
		//  X-specific structure
		//! Set the delegate to use for syntax highlighting.
		//! X11 version only
		void SetDelegateOnSyntaxHighlight (
			fastdelegate::FastDelegate<void(XeTextSyntaxParseCallbackData*)> delegate
			);
#endif

		//! Set equal tab stops 
		//! Call this function to set the tab stops in a multiple-line edit control. 
		//! When text is copied to a multiple-line edit control, any tab character in 
		//! the text will cause space to be generated up to the next tab stop.
		//!	The default is one tab stop every 8 characters.
		void SetTabStops (
			int cxEachStop		//!< tab stops are to be set at every cxEachStop dialog units 
			);

		//! Set current string value with validation.
		//! OnValidate() will be called and if valid, the string will be updated.
		//! No comparison is made with the current string value.
		void SetValue (
			const MIUNICODE* string,					//!< New string to set
			bool notify = true						//!< Call OnChangeValue() if string actually updated
			);

		//! Show the cursor (a.k.a. caret)
		void ShowCursor ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		enum CMDEDIT {
			CMDEDIT_Cut =		1001,
			CMDEDIT_Copy,
			CMDEDIT_Paste,
			CMDEDIT_EraseSelection,
			CMDEDIT_SelectAll,
			};

		MISTRING m_value;
		DELEGATE_VOID_MISTRING m_DelegateOnHyperLink;
		fastdelegate::FastDelegate<void(XeTextSyntaxParseCallbackData*)> m_DelegateOnSyntaxHighlight;
		DELEGATE_ONCURSORMOTION m_DelegateOnCursorMotion;

		static void CB_HyperLink (Widget, CTRL_EDIT_TEXT*, void*);
		static void CB_SyntaxHighlight (Widget, CTRL_EDIT_TEXT*, void*);
		static void CB_CursorMotion (Widget, CTRL_EDIT_TEXT*, void*);

		virtual void v_UpdateValue (MISTRING& newvalue);

		CTRL_EDIT_TEXT (const CTRL_EDIT_TEXT&);
		CTRL_EDIT_TEXT& operator= (const CTRL_EDIT_TEXT&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::CTRL_EDIT_TEXT::FLAGS)
#endif

//===================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_EDIT_H
