/**
 * \file <mgui/toolbar.h>
 * \brief MGUI::CTRL_TOOLBAR definitions
 *
 * \if NODOC
 * $Id: toolbar.h_v 1.15 2010/03/01 22:58:39 mju Exp $
 *
 * $Log: toolbar.h_v $
 * Revision 1.15  2010/03/01 22:58:39  mju
 * Remove textid variants.
 *
 * Revision 1.14  2008/04/28 16:36:03  mju
 * Make dlg_customize a friend.
 *
 * Revision 1.13  2008/02/07 15:09:10  mju
 * Add delete().
 *
 * Revision 1.12  2008/02/06 21:02:11  mju
 * Change createMenu return type to void.
 *
 * Revision 1.11  2008/02/04 21:24:46  mju
 * Add deleteAll.
 *
 * Revision 1.10  2007/12/17 15:29:50  mju
 * Add createitem_Hidden and setVisible/ID.
 *
 * Revision 1.9  2007/12/07 18:21:40  mju
 * Add creatitem_Check/RadioOn values and createitem_Check/RadioSet methods.
 *
 * Revision 1.8  2007/11/26 14:07:14  mju
 * Add addSpaceFixed.
 *
 * Revision 1.7  2007/11/21 22:41:56  mju
 * Add createitem_RadioFirst.
 *
 * Revision 1.6  2007/11/21 17:38:57  mju
 * Allow bitwise ops for createitem flags.
 *
 * Revision 1.5  2007/11/08 14:23:24  mju
 * Add setDelegate.
 *
 * Revision 1.4  2007/08/30 13:12:37  mju
 * Add getWidgetFromID.
 *
 * Revision 1.3  2007/03/07 22:20:59  mju
 * Add get/setToggleState...
 *
 * Revision 1.2  2005/10/31 16:47:39  mju
 * Add setEnabledItem/ID methods.
 *
 * Revision 1.1  2005/10/28 20:40:21  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TOOLBAR_H
#define	INC_MGUI_TOOLBAR_H

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

#ifndef	INC_MGUI_MENU_H
#include <mgui/menu.h>
#endif

namespace MGUI {

class DLG_CUSTOMIZE;

//=====================================================================================================================

class CTRL_TOOLBAR : public CTRL {
	public:

		//! Flags for use when adding items.
		enum CREATEITEM {
			CREATEITEM_Default =			MENU::CREATEITEM_Default,
			CREATEITEM_Check =			MENU::CREATEITEM_Check,				//!< Create with check style, cannot be combined with CREATEITEM_Radio
			CREATEITEM_CheckOn =			MENU::CREATEITEM_CheckOn,			//!< Create with check style, initially toggled on
			CREATEITEM_Radio =			MENU::CREATEITEM_Radio,				//!< Create with radio style, cannot be combined with CREATEITEM_Check
			CREATEITEM_RadioOn =			MENU::CREATEITEM_RadioOn,			//!< Create with radio style, initially toggled on
			CREATEITEM_RadioFirst =		MENU::CREATEITEM_RadioFirst,		//!< First item in radio group, implies radio style, only need if multiple radio groups
			CREATEITEM_RadioFirstOn =	MENU::CREATEITEM_RadioFirstOn,	//!< First item in new radio group, initially toggled on
			CREATEITEM_ToggleOn =		MENU::CREATEITEM_ToggleOn,			//!< Create initially toggled on, requires CREATEITEM_Check or CREATEITEM_Radio
			CREATEITEM_Disabled =		MENU::CREATEITEM_Disabled,			//!< Create initially disabled
			CREATEITEM_NoDelegate =		MENU::CREATEITEM_NoDelegate,		//!< Create item with no delegate, use when menu has default delegate if want label
			CREATEITEM_MI_Internal =	MENU::CREATEITEM_MI_Internal,		//!< Only create if MicroImages internal license
			CREATEITEM_MI_Programmer =	MENU::CREATEITEM_MI_Programmer,	//!< Only create if MicroImages programmer license
			CREATEITEM_Hidden =			MENU::CREATEITEM_Hidden				//!< Create item initially hidden
			};

		//! Predefined item handles.
		enum ITEMHND {
			ITEMHND_None = MENU::ITEMHND_None,
			ITEMHND_Last = MENU::ITEMHND_Last
			};

		//! Delegate for selection from button.
		//! Values passed to delegate are ID and toggle state.  If item is not a toggle the
		//! state will always be 'true'.
		typedef fastdelegate::FastDelegate<void(MGUI::ID,bool)> DELEGATE;

		//! Convenience method to get CREATEITEM flag for check item with control of whether initially on.
		static CREATEITEM CREATEITEM_CheckSet (
			bool ToggleOn
			) { return (ToggleOn ? CREATEITEM_CheckOn : CREATEITEM_Check); }

		//! Convenience method to get CREATEITEM flag for radio item with control of whether initially on.
		static CREATEITEM CREATEITEM_RadioSet (
			bool ToggleOn,
			bool RadioFirst = false
			) { return (RadioFirst ? (ToggleOn ? CREATEITEM_RadioFirstOn : CREATEITEM_RadioFirst) : (ToggleOn ? CREATEITEM_RadioOn : CREATEITEM_Radio)); }

		//! Default constructor.
		CTRL_TOOLBAR ();

		//! Destructor.
		virtual ~CTRL_TOOLBAR ();

		//! Add an item to the toolbar, using default delegate.
		//! Must be called before Create().
		//! @return Handle to item for subsequent reference, will be < 0 if error
		ITEMHND Add (
			ICONID iconid,							//!< Icon ID
			const MISTRING& label,				//!< Label for item
			ID id,									//!< Application-defined ID to be passed to default delegate
			CREATEITEM createflags = CREATEITEM_Default,
			UINT32 icid = 0						//!< Interface customization ID
			);

		//! Add an item to the toolbar, using specified delegate.
		//! Must be called before Create().
		//! @return Handle to item for subsequent reference, will be < 0 if error
		ITEMHND Add (
			ICONID iconid,							//!< Icon ID
			const MISTRING& label,				//!< Label for item
			MGUI::MENU::DELEGATE delegate,	//!< Delegate to call
			ID id = 0,								//!< Optional application-defined ID to be passed to delegate
			CREATEITEM createflags = CREATEITEM_Default,
			UINT32 icid = 0						//!< Interface customization ID
			);

		//! Add a item with menu to toolbar.
		//! Must be called before Create().
		//! @return Handle to item for subsequent reference, will be < 0 if error
		ITEMHND Add (
			ICONID iconid,							//!< Icon ID
			const MISTRING& label,				//!< Label
			MENU& menu,								//!< Menu for item
			CREATEITEM createflags = CREATEITEM_Default,
			UINT32 icid = 0						//!< Interface customization ID
			);

		//! Add separator to toolbar.
		//! Must be called before Create().
		void AddSeparator (
			);

		//! Add fixed space to toolbar.
		//! Must be called before Create().
		void AddSpaceFixed (
			int space								//!< Space in pixels
			);

		//! Create control.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			LAYOUT_ORIENTATION orientation = LAYOUT_ORIENTATION_Horizontal,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize
			);

		//! Delete item from toolbar.
		void Delete (
			ITEMHND hndItem						//!< Handle returned by Add()
			);

		//! Delete all items from toolbar.
		void DeleteAll (
			);

		//! Get whether item is toggled on using handle.
		bool GetToggleStateItem (
			ITEMHND hndItem						//!< Handle returned by Add()
			) const;

		//! Get whether item is toggled on using ID.
		bool GetToggleStateID (
			ID id										//!< ID of item(s) to enable
			) const;

		//! Set delegate for existing item ID.
		void SetDelegate (
			ID id,									//!< ID of item
			DELEGATE delegate						//!< Delegate to set
			) { m_menu.SetDelegate(id,delegate); }

		//! Set default delegate for use with Add() when no delegate specified.
		//! If used, must be called before Add().
		void SetDftDelegate (
			DELEGATE delegate
			);

		//! Set whether item is enabled using handle.
		void SetEnabledItem (
			ITEMHND hndItem,						//!< Handle returned by Add()
			bool enabled							//!< true to enable, false to disable
			);

		//! Set whether item is enabled using ID.
		void SetEnabledID (
			ID id,									//!< ID of item(s) to enable
			bool enabled							//!< true to enable, false to disable
			);

		//! Set whether item is toggled on using handle.
		void SetToggleStateItem (
			ITEMHND hndItem,						//!< Handle returned by Add()
			bool ToggleOn,							//!< true to toggle on, false to toggle off
			bool notify								//!< Send notification via delegate if true
			);

		//! Set whether item is toggled on using ID.
		void SetToggleStateID (
			ID id,									//!< ID of item(s) to enable
			bool ToggleOn,							//!< true to toggle on, false to toggle off
			bool notify								//!< Send notification via delegate if true
			);

		//! Set whether item is visible using handle.
		void SetVisibleItem (
			ITEMHND hndItem,						//!< Handle returned by Add()
			bool visible							//!< true to show, false to hide
			);

		//! Set whether item is visible using ID.
		void SetVisibleID (
			ID id,									//!< ID of item(s) to show
			bool visible							//!< true to enable, false to hide
			);

	#if defined(X_NATIVE) && !defined(GENERATING_DOXYGEN_OUTPUT)
		// Get X Widget from item ID.
		// For internal MGUI use only.
		Widget GetWidgetFromID (
			ID id
			) const { return (m_menu.GetWidgetFromID(id)); }
	#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		friend class MGUI::DLG_CUSTOMIZE;

		CTRL_TOOLBAR (const CTRL_TOOLBAR&);
		CTRL_TOOLBAR& operator= (const CTRL_TOOLBAR&);

		MGUI::MENU m_menu;
		bool m_MenuCreated;

		void CreateMenu ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CTRL_TOOLBAR::CREATEITEM)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLBAR_H
