/***************************************************************************
 * \file mi32/httpmsg.h
 * \brief HTTPMESSAGE definitions.
 *
 * \if NODOC
 * $Log: httpmsg.h_v $
 * Revision 1.20  2012/04/26 19:36:34  mju
 * Fix status_TemporaryRedirect value.
 *
 * Revision 1.19  2012/04/26 17:16:45  mju
 * Add getStatusText and fix incorrect status number.
 *
 * Revision 1.18  2012/04/09 14:37:29  mju
 * Clean up a bit, remove unnecessary hdrs.
 *
 * Revision 1.17  2011/10/26 12:50:56  mju
 * Remove redundant hdr.
 *
 * Revision 1.15  2011/04/08 20:24:54  ldyas
 * added convenience method for appending to the body
 *
 * Revision 1.14  2008/05/01 19:17:03  scowan
 * Use escapes URI if available.
 *
 * Revision 1.12  2007/04/12 17:29:19  mju
 * Added op=.
 *
 * Revision 1.11  2007/02/08 23:16:25  dwilliss
 * Added GetDestFile() method
 *
 * Revision 1.10  2007/01/24 15:05:48  dwilliss
 * Added DetachData method.
 * More documentation
 *
 * Revision 1.9  2007/01/18 21:43:32  dwilliss
 * Added GetNumBytesExpected()
 *
 * Revision 1.6  2006/02/23 20:16:58  ldyas
 * adjusted for multipart posting.
 *
 * Revision 1.3  2004/01/09 18:36:02  dwilliss
 * Added methods to get/set content char encoding
 *
 * Revision 1.1  2004/01/02 22:58:03  dwilliss
 * Initial revision
 *
 * \endif
 **************************************************************************/

#ifndef INC_MI32_HTTPMSG_H
#define INC_MI32_HTTPMSG_H

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif


#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class DATETIME;
class FILEPATH;
class STRUTF8;
class XMLDOC;
#endif


//! Class to encapuselate an HTTP message.
//!
//! When recieving a message, this class holds both the headers and the message
//! body in its raw form, which you can retrieve as raw data, MISTRING, STRUTF8,
//! or an XMLDOC.
//!
//! When sending a message, you can store the body in the HTTPMESSAGE or you
//! can use HTTPMESSAGE to hold just the headers and use one of the many
//! versions of HTTPCLIENT::SendMessage() which take the body as a seperate
//! parameter.
class CLASSLIBEXPORT HTTPMESSAGE {
	public:

		enum METHOD {
			METHOD_REPLY = 0,
			METHOD_OPTIONS	 = 1,
			METHOD_GET,
			METHOD_HEAD,
			METHOD_POST,
			METHOD_PUT,
			METHOD_DELETE,
			METHOD_TRACE,
			METHOD_CONNECT,
			METHOD_NUM_METHODS = 16	// If we ever get a method #16, we have a problem
			};

		//! HTTP status codes as defined by RFC-2616
		//!	1xx	Informational
		//!	2xx	Successful
		//!	3xx	Redirection
		//!	4xx	Client error
		//!	5xx	Server error
		enum STATUS {
			STATUS_EmptyMessage =						0,

			STATUS_Continue = 							100,

			STATUS_OK		 = 							200,
			STATUS_Created	 = 							201,
			STATUS_Accepted	 = 						202,
			STATUS_NonAuthoritativeInfo = 			203,
			STATUS_NoContent	 = 						204,
			STATUS_ResetContent	 = 					205,
			STATUS_PartialContent	 = 				206,

			STATUS_MultipleChoices =				 	300, 
			STATUS_MovedPermanently = 					301,
			STATUS_Found = 								302,
			STATUS_SeeOther = 							303,
			STATUS_NotModified = 						304,
			STATUS_UseProxy = 							305,
															// 306 unused
			STATUS_TemporaryRedirect = 				307,

			STATUS_BadRequest = 							400,
			STATUS_Unauthorized = 						401,	//!< Password required
			STATUS_PaymentRequired = 					402,
			STATUS_Forbidden = 							403,	//!< No permission
			STATUS_NotFound = 							404,
			STATUS_MethodNotAllowed = 					405,
			STATUS_NotAcceptable = 						406,
			STATUS_ProxyAuthenticationRequired = 	407,
			STATUS_RequestTimeOut = 					408,
			STATUS_Conflict = 							409,
			STATUS_Gone = 									410,
			STATUS_LengthRequired = 					411,
			STATUS_PreconditionFailed = 				412,
			STATUS_RequestEntityTooLarge = 		  	413,
			STATUS_RequestURITooLarge = 			  	414,
			STATUS_UnsupportedMediaType = 		  	415,
			STATUS_RequestedRangeNotSatisfiable = 	416,
			STATUS_ExpectationFailed = 			  	417,

			STATUS_InternalServerError = 				500,
			STATUS_NotImplemented = 					501,
			STATUS_BadGateway = 							502,
			STATUS_ServiceUnavailable = 				503,
			STATUS_GatewayTimeOut = 					504,
			STATUS_HTTPVersionNotSupported = 		505
			};

		//! Get localized text for status.
		static MISTRING GetStatusText (
			STATUS status
			);

		//! Default constructor.
		HTTPMESSAGE ();

		//! Copy constructor
		HTTPMESSAGE (
			const HTTPMESSAGE& rhs
			);

		~HTTPMESSAGE ();

		//! Assignment.
		HTTPMESSAGE& operator= (
			const HTTPMESSAGE& rhs
			);

		//! Add an additional header not covered by the other Set methods.
		//! If you try to add a header that already exists in the message,
		//! it will replace the existing header
		void AddHeader (
			const STRUTF8& header
			);

		//! Append data to body of message.
		//! This method is used to allow setting the body of a message to be sent generally by post.
		ERRVALUE AppendToBody (
			const void* data,
			int numbytes
			);
		
		//! Append MISTRING to body of message.
		ERRVALUE AppendToBody (
			const MISTRING& data
			);

		//! Clear the destination file.
		//! If the HTTPMESSAGE was last used to recieve directly to a file, this 
		//! method resets it so that the HTTPMESSAGE can be reused without 
		//! overriting the same file.
		//! See SetDestFile() for details
		void ClearDestFile ();

		//! Close the destination file.
		//! See SetDestFile() for details
		void CloseDestFile ();

		//! Detach and return a pointer to the raw message body data.
		//! This method calls Detach on the body data buffer and returns the
		//! pointer.  You should call GetNumBytes() before calling this method.
		//! This method obviously won't work if you've redirected the body to
		//! a file by calling SetDestFile().
		//! The caller is responsible for freeing the buffer.
		UINT8* DetachData ();

		//! Get the escaped URI for the request.
		//! This is like GetRequestURI, but escapes the URI by the rules of
		//! RFC1738.
		void GenerateEscapedRequestURI (
			STRUTF8& uri
			) const;

		//! Get the message body as an MISTRING
		ERRVALUE GetContent (
			MISTRING& mistr
			) const;

		//! Get the message body as a STRUTF8
		ERRVALUE GetContent (
			STRUTF8& mistr
			) const;

		//! Get the message body as an XML document
		//!  This method works with the following ContentTypes
		//! * text/xml	
		//! * text/html
		//! * text/plain - will create a single text node
		//! * multipart/form-data - This is how HTML forms submit via POST
		//!   The document will have a "control" node for each
		//!	value.  Each node will have a "name" attribute which holds
		//!	the name of the control from the form.
		//! * application/x-www-form-urlencoded - This is how HTML forms
		//!	submit via GET. XML document format is the same as for
		//!	multipart/form-data
		ERRVALUE GetContent (
			XMLDOC& mistr
			) const;

		//! Get the content character encoding.
		//! Note, if no encoding is specified or is meaningless (such as for
		//! binary data), the encoding will be assumed to be UTF8
		CHAR_ENCODING GetContentCharEncoding () const;

		//! Get the content character encoding.
		//! The encoding string is defined by some RFC that I have to look up.
		//! \return true if the encoding was set, false if not
		bool GetContentCharEncoding (
			STRUTF8& str
			) const;

		//! Get the value of the Content-Type in the HTTP header.
		//! This will only be the MIME type.  If the Content-Type specified a
		//! charencoding, it will have been parsed and you should use 
		//! GetContentCharEncoding() to retrieve it.
		const char* GetContentType () const;
	
		//! Get a pointer to the body data (const)
		//! Note: there is also a DetachData() method which will detach the
		//! buffer from the HTTPMESSAGE, eliminating the need to copy it.
		//! This method obviously won't work if you've redirected the body to
		//! a file by calling SetDestFile().
		const UINT8* GetData (
			) const;

#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Get a pointer to the body data (non-const).
		//! This should be private.
		UINT8* GetData (
			);
#endif

		//! Get the destination file.
		//! If the HTTPMESSAGE is set to retrieve directly to a file, this will
		//! return the FILEPATH it's set to retrieve to.  If HTTPMESSAGE is not
		//! set to retrieve directly to a file, the returned FILEPATH will be
		//! empty (IsEmpty() will return true).
		const FILEPATH& GetDestFile () const;

		//! Get the escaped URI of the request, result is an empty string if it does not exist.
		const STRUTF8& GetEscapedRequestURI () const;

		//! Get a list of headers that were not parsed
		const MILIST<STRUTF8>& GetExtraHeaders (
			) const;

		// Get the value of a header entry
		// Note: you cannot get the date/time entries this way
		// \param key The header to get, not including the colon
		// \return The value of the header or NULL if not found
		const char* GetHeaderString (
			const char* key
			) const;

		//! Returns true if the request had the Connection: Keep-Alive header.
		bool GetKeepAlive () const;

		//! Returns the time from the HTTP header when the object was retrieved.
		//! Note, this will be the time on the server, not local machine time.
		const DATETIME& GetMessageTime (
			) const;

		//! Get the multipart boundary string for inclusion 
		//! in the Content Type Header
		const STRUTF8& GetMultiPartBoundary () const;

		//! Get the number of bytes in the body
		int GetNumBytes (
			) const;

		//! Get the number of bytes in the body.
		//! If the message has been fully read from the server, this will be the
		//! same as GetNumBytes.  If the message has only been partly read, 
		//! GetNumBytes() will return the size read so far and 
		//! GetNumBytesExpected() will return the expected size.
		int GetNumBytesExpected (
			) const;

		METHOD GetRequestMethod () const;

		//! Get the URI of the request
		const STRUTF8& GetRequestURI (
			) const;

		//! Get the status code from the HTTP header.
		STATUS GetStatusCode () const;

		//! Get the status message from the HTTP header.
		const STRUTF8& GetStatusMessage () const;

		//! Get the object's last modified time
		const DATETIME& GetTimeLastModified () const;

		//! Return true if a CHAR_ENCODING has been set.
		bool HasCharEncoding () const;

		//! Set by HTTPCLIENT if the message is from our cache
		bool IsCachedResult () const;

		//! Returns true if this is a HEAD request
		//! This is just a shortcut because you need it a lot and this is
		//! more compact and readable than
		//!		request.GetRequestMethod() == HTTPMESSAGE::METHOD_HEAD
		bool IsHeadRequest () const;

		//! Determine if we're writing directly to a file.
		bool IsDirectToFile () const;

		//! Reserve memory for the buffer for the body of the message
		//! This also sets the NumBytesExpected()
		ERRVALUE Reserve (
			int size
			);

		//! Reset the message to an initial state
		void Reset ();

		//! Resize the buffer for the body of the message
		ERRVALUE Resize (
			int size
			);

		//! Set the content character encoding.
		void SetContentCharEncoding (
			CHAR_ENCODING encoding
			);

		//! Set the content character encoding.
		//! The encoding string is defined by some RFC. 
		//! (TO DO: look up the RFC).
		void SetContentCharEncoding (
			const char* encoding
			);

		//! Set the value of the Content-Type in the HTTP header.
		//! \param ContentType any valid MIME type
		void SetContentType (
			const char* ContentType
			);

		//! Set the filepath to write the message body to.
		//! Set this before recieving a message if you'd rather have the message
		//! body writen directly to a file.
		//! To cancel this behaviour, call ClearDestFile();
		void SetDestFile (
			const FILEPATH& filepath
			);
		
		//! Set by HTTPCLIENT if filling in a reply message from the cache.
		void SetIsCachedResult (
			bool bCachedResult
			);

		//! Set the Connection: header to Keep-Alive. 
		void SetKeepAlive (
			bool bKeepAlive = true
			);
		
		//! Set the Date in the HTTP header.
		//! This is used when recieving a message to record the time in the
		//!	recieved headers.
		//! When sending a message, we always use the current time.
		void SetMessageTime (
			const DATETIME& dts
			);

		//! Setup the message as an HTTP request
		void SetRequest (
			METHOD method,
			const STRUTF8& uri,
			bool bIsEscaped = false //!< Pass true if uri is escaped as per RFC1738 (default = false)
			);

		//! This is used by a server when sending a reply.
		void SetStatusCode (
			STATUS status
			);

		//! Set the status message for the HTTP header.
		//! This is used by a server when sending a reply.
		//! Note, this one might go away.  The message should be looked-up from
		//! the status code.
		void SetStatusMessage (
			const char* msg
			);

		//! Set the Last-Modified in the HTTP header.
		//! This is used by a server when sending a reply.
		void SetTimeLastModified (
			const DATETIME& dts
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_priv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#undef CLASSLIBEXPORT

#endif // INC_MI32_HTTPMSG_H
