/**
 * \file mi32/interlockedint32.h
 * \brief Definition for interlocked INT32 class
 *
 * \if NODOC
 * $Id: interlockedint32.h_v 1.21 2008/09/02 17:55:02 mju Exp $
 *
 * $Log: interlockedint32.h_v $
 * Revision 1.21  2008/09/02 17:55:02  mju
 * Fix doxygen warning.
 *
 * Revision 1.20  2008/08/07 20:37:53  scowan
 * Undefine class lib export at end of file.
 *
 * Revision 1.19  2007/10/18 15:56:24  scowan
 * Added code example to docs.
 *
 * Revision 1.18  2007/09/07 16:44:50  scowan
 * Arrggg...
 *
 * Revision 1.17  2007/09/07 16:23:53  scowan
 * More attempted fixes.
 *
 * Revision 1.16  2007/09/07 15:32:14  scowan
 * Attempted linux 64 fix.
 *
 * Revision 1.15  2007/09/06 22:18:38  scowan
 * Added inclusion guard for linux 64.
 *
 * Revision 1.14  2007/09/06 22:10:25  scowan
 * More linux fixes.
 *
 * Revision 1.13  2007/09/06 21:57:23  scowan
 * More fixes.
 *
 * Revision 1.12  2007/09/06 21:48:41  scowan
 * Maybe fixed for linux32 build.
 *
 * Revision 1.11  2007/09/06 21:35:45  scowan
 * Attempted fix for linux 64 builds.
 *
 * Revision 1.10  2007/09/06 21:02:17  scowan
 * Updated for sun.
 *
 * Revision 1.9  2007/08/24 16:47:48  scowan
 * Windows made me un-inline the methods, grrr....
 *
 * Revision 1.8  2007/08/24 14:36:13  scowan
 * Fixed sun issue with new method.
 *
 * Revision 1.7  2007/08/23 21:56:53  scowan
 * Added compareAdnExchange method.
 *
 * Revision 1.6  2007/08/23 16:42:56  scowan
 * More sun fixes.
 *
 * Revision 1.5  2007/08/23 15:04:10  scowan
 * Fixed sun stuff, I hope.
 *
 * Revision 1.4  2007/08/23 14:52:44  scowan
 * More mac fixes.
 *
 * Revision 1.3  2007/08/22 23:06:19  scowan
 * Fixed for sun and more fixes for mac.
 *
 * Revision 1.2  2007/08/22 22:48:10  scowan
 * Fixed for mac and linux.
 *
 * Revision 1.1  2007/08/22 21:47:21  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_INTERLOCKEDINT32_H
#define	INC_MI32_INTERLOCKEDINT32_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

namespace MITHREAD {

//! Interlocking in multi-core systems is used to ensure that the per-core cache agrees with all of the other per-core
//! caches and main memory.  On multi-core systems, there is a race condition with a shared variable that if the variable 
//! is in the core cache and not in the other core caches, then another thread will get the wrong value.  This class 
//! wraps the UNIX and Windows INT32 interlocked API's to enforce the synchronization of INT32's.  Most of the basic integer
//! operators that would modify the INT32 value is available in this class.
//! Since this synchronization is done in user mode and not by the OS, it is significantly faster than using mutexes and 
//! other synchronization operations.  
//! @code
//!
//!	ERRVALUE MICACHE::Select (
//!		INT32 ItemID,
//!		ITEMINFO& iteminfo,
//!		SELECTFLAGS flags,
//!		void* UserDataPtr
//!		) {
//!		...
//!		if (m_CacheAccessCount < 0) return (SetErrPosnC(EInvalidCacheResize));		// This thread is attempting to access when another thread is resizing
//!		m_CacheAccessCount ++;		// After incrementing, all threads will see the same value for m_CacheAccessCount
//!		...
//!		return (0);
//!		}
//!
//!
//!	void MICACHE::ITEMINFO::~ITEMINFO (
//!		) {
//!		...
//!		m_CacheAccessCount --;		// After decrementing, all threads will see the same value for m_CacheAccessCount
//!		...
//!		}
//!
//!
//!	ERRVALUE MICACHEPRIV::Resize (
//!		INT32 NumCache,
//!		INT32 NumHash
//!		) {
//!		...
//!		// If m_CacheAccessCount > 0, then the return value will != 0 and the error will be sent.
//!		// That means that someone other thread is accessing the cache.  If m_CacheAccessCount == 0, 
//!		// then m_CacheAccessCount will be assigned -1, which the MICACHE::Select() code above is checking
//!		// for and returns an error if the cache is begin resized when another thread is accessing the cache.
//!		if (m_CacheAccessCount.CompareAndExchange(-1, 0) != 0) return (SetErrPosnC(EInvalidCacheResize));		// Some other thread is attempting to resize when other threads are accessing the cache
//!
//!		...
//!		// Allow the cache to be accessable again to the other threads.
//!		m_CacheAccessCount = 0;		// After assignment, all threads will see the same value for m_CacheAccessCount
//!		...
//!		return (0);
//!		}
//!
//! @endcode
class CLASSLIBEXPORT INTERLOCKEDINT32 {
	public:
		//! Default ctor, initialize to zero
		INTERLOCKEDINT32 (
			);
		
		//! Copy ctor, init to zero then do an Interlocked Exchange 
		INTERLOCKEDINT32 (
			const INTERLOCKEDINT32& rhs
			);
	
		//! INT32 to this ctor, init to zero then do an Interlocked Exchange 
		INTERLOCKEDINT32 (
			INT32 value
			);
	
		//! Default dtor
		~INTERLOCKEDINT32 (
			);
			
		//! Assignment operator, do an Interlocked Exchange 
		//! @return 'this'.
		INTERLOCKEDINT32& operator= (
			const INTERLOCKEDINT32& rhs
			);

		//! Pre-increment operator, do an Interlocked Increment, return this
		//! @return 'this'.
		INTERLOCKEDINT32& operator++ (
			);
	
		//! Post-increment operator, do an Interlocked Increment, return the value from the intrinsic function.
		//! @return Previous value stored in 'this'.
		INT32 operator++ (int
			);
	
		//! Pre-decrement operator, do an Interlocked Decrement.
		//! @return 'this'.
		INTERLOCKEDINT32& operator-- (
			);
	
		//! Post-decrement operator, do an Interlocked Decrement, return the value from the intrinsic function.
		//! @return Previous value stored in 'this'.
		INT32 operator-- (int
			);
	
		//! Addition operator, do an Interlocked Add.
		//! @return 'this'.
		INTERLOCKEDINT32& operator+= (
			INT32 with
			);

		//! Subtraction operator, do an Interlocked Add.
		//! @return 'this'.
		INTERLOCKEDINT32& operator-= (
			INT32 with
			);
	
		//! AND operator.
		//! @return 'this'.
		INTERLOCKEDINT32& operator&= (
			INT32 with
			);
	
		//! OR operator.
		//! @return 'this'.
		INTERLOCKEDINT32& operator|= (
			INT32 with
			);
	
		//! XOR operator.
		//! @return 'this'.
		INTERLOCKEDINT32& operator^= (
			INT32 with
			);
	
		//! Cast operator to get the INT32 value stored
		//! The compile condition is because Solaris 9 does not make available its atomic functions public and
		//! to attempt to maintain the fastest possible code for the other platforms.
		//! @return Signed integer value of 'this'
		#if defined(SUN) || defined(LINUX)
		operator INT32 (
			) const;
		#else
		operator INT32 (
			) const { return (m_Value); }
		#endif

		//! Performs the following operation atomically
		//! INT32 retval = 'this'; if ('this' == TestValue) 'this' = NewValue; return (retval);
		//! @return Value of 'this' BEFORE the call.
		INT32 CompareAndExchange (
			INT32 NewValue,
			INT32 TestValue
			);
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		#if defined(MAC_OSX)
			INT32 m_Value;
		#elif defined(SUN) || defined(LINUX)
			class PRIV;
			PRIV* m_pPriv;			//!< Unlikely to have an updated SUN machine with proper code, therefore we use this slow method
		#else
			INT32 volatile m_Value;
		#endif
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of MITHREAD namespace

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_INTERLOCKEDINT32_H
