/**
 * \file mi32/lidarlaspoint.h
 * \brief LIDAR::LAS POINTARRAY and POINTRECORD definitions.
 *
 * \if NODOC
 * $Id: lidarlaspoint.h_v 1.8 2012/09/18 13:43:55 mju Exp $
 *
 * $Log: lidarlaspoint.h_v $
 * Revision 1.8  2012/09/18 13:43:55  mju
 * Attemp dox fix.
 *
 * Revision 1.7  2012/06/18 14:57:56  mju
 * Add get/setClassFlags.
 *
 * Revision 1.6  2012/06/18 14:15:49  mju
 * Add getColor4 and getNumColorValues.
 *
 * Revision 1.5  2012/06/14 21:32:42  mju
 * Don't inline methods where differs for LAS 1.4
 *
 * Revision 1.4  2011/09/06 14:45:34  mju
 * Add recordSize parm to setFormat and remove priv member.
 *
 * Revision 1.3  2011/07/13 19:21:37  mju
 * Add delegate_bool_pointrecord.
 *
 * Revision 1.2  2011/06/24 21:45:28  mju
 * Fix for byteswapped.
 *
 * Revision 1.1  2011/06/24 21:10:34  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_LIDARLASPOINT_H
#define	INC_LIDARLASPOINT_H

#ifndef	INC_MI32_LIDARPOINT32_H
#include <mi32/lidarpoint32.h>
#endif

#ifndef	INC_MI32_LIDARENUMS_H
#include <mi32/lidarenums.h>
#endif

#ifndef	INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif


#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
struct COLOR;
class FILE_RAW;
#endif


namespace LIDAR {
namespace LAS {

//=====================================================================================================================

//! Array of LAS point records.
class CLASSLIBEXPORT POINTARRAY : public POINTARRAY32 {
	public:

		//! Constructor.
		POINTARRAY ();

		//! Destructor.
		~POINTARRAY ();

		UINT8 GetFormat () const
			{ return (m_Format); }

		UINT16 GetOffsetColor () const
			{ return (m_OffsetColor); }

		UINT16 GetOffsetSourceID () const
			{ return (m_OffsetSourceID); }

		UINT16 GetOffsetTime () const
			{ return (m_OffsetTime); }

		UINT16 GetOffsetWave () const
			{ return (m_OffsetWave); }

		//! Set record format.
		void SetFormat (
			UINT8 format,			//!< Record format
			UINT16 RecordSize		//!< Record size in bytes, 0 for default
			);

		#ifdef BYTEORDER_HiLo
		void SwapBytes ();
		#endif

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		UINT16 m_OffsetSourceID;
		UINT16 m_OffsetColor;
		UINT16 m_OffsetTime;
		UINT16 m_OffsetWave;
		UINT8 m_Format;
		#endif
	};

//=====================================================================================================================

//! Point record for LAS format.
class CLASSLIBEXPORT POINTRECORD : public POINTRECORD32 {
	public:

		enum CLASSFLAGS {
			CLASSFLAG_None =				0,
			CLASSFLAG_Synthetic =		0x01,
			CLASSFLAG_KeyPoint =			0x02,
			CLASSFLAG_Deleted =			0x04,
			CLASSFLAG_Overlap =			0x08,
			};

		//! Default constructor, creates NULL record.
		POINTRECORD (
			) { }

		//! Copy constructor.
		POINTRECORD (
			const POINTRECORD& rhs
			) : POINTRECORD32(rhs) { }

		//! Construct and attach to array.
		POINTRECORD (
			POINTARRAY& Array
			) : POINTRECORD32(Array) { }

		//! Construct and attach to array with index.
		POINTRECORD (
			POINTARRAY& Array,
			UINT32 ArrayIdx
			) : POINTRECORD32(Array,ArrayIdx) { }

		//! Destructor.
		~POINTRECORD (
			) { }

		//! Copy contents of one point record to another.
		//! Records do not have to be associated with the same file.
		void CopyFrom (
			const POINTRECORD& src
			);

		//! Get classification flag bits (Synthetic, Key-point, Deleted, Overlap).
		UINT8 GetClassFlags () const;

		//! Get full 'class', including 'Classification', 'Synthetic', 'Key-point', 'Deleted', 'Overlap' status bits.
		//! This is more efficient than accessing multiple individual parts.
		//! Bits 0-7 are 'Classification, 8 is 'Synthetic', 9 is 'Key-point', 10 is 'Deleted', 11 is 'Overlap'
		DEPRECATED UINT16 GetClassFull () const;

		//! Get classification.
		//! @return Classification, range depends on point format.
		UINT8 GetClassification () const;

		//! Get 'color' value from record.
		void GetColor (
			COLOR& color					//!< Color returned, left unchanged if no color field
			) const;

		//! Get 4 color values, typically Red, Green, Blue, Near infrared.
		void GetColor4 (
			UINT16 *values					//!< Up to 4 color values returned, array must be large enough to hold 4 values, unused values left unchanged
			) const;

		//! Get 'intensity' value.
		UINT16 GetIntensity () const
			{ return (*reinterpret_cast<const UINT16*>(GetData()+OFFSET_Intensity)); }

		//! Get number of color values.
		UINT16 GetNumColorValues () const;

		//! Get return count (number of returns for pulse).
		UINT8 GetReturnCount () const;

		//! Get return number.
		UINT8 GetReturnNumber () const;

		//! Get point return "type".
		POINTRETURNTYPE GetReturnType () const;

		//! Get return values (packed).
		//! @return Combined return count (high 4 bits) and number (low 4 bits).
		UINT8 GetReturnValuesPacked () const;

		//! Get 'scan angle'.
		float GetScanAngle () const;

		//! Get 'scan direction'.
		INT8 GetScanDirection () const;

		//! Get 'source ID'.
		UINT16 GetSourceID () const
			{ return (*reinterpret_cast<const UINT16*>(GetData()+GetOffsetSourceID())); }

		//! Get 'GPS Time'.
		//! @return Time for point interpretation depends on GPS Time Type of file, will return 0 if no time available.
		double GetTime () const;

		//! Get 'user data value'.
		UINT8 GetUserData () const
			{ return (GetData()[OFFSET_UserData]); }

		UINT8 GetWaveDescIdx () const;

		UINT64 GetWaveOffset () const;

		float GetWavePointLocation () const;

		UINT32 GetWaveSize () const;

		float GetWaveXt () const;

		float GetWaveYt () const;

		float GetWaveZt () const;

		bool HasColor () const
			{ return (GetOffsetColor() != 0); }

		bool HasTime () const
			{ return (GetOffsetTime() != 0); }

		bool HasWave () const
			{ return (GetOffsetWave() != 0); }

		//! Determine if point has 'deleted' attribute set.
		bool IsDeleted () const;

		//! Determine if point is at end of sweep (edge of flight line).
		bool IsEndOfSweep () const;
			
		//! Determine if point is 'first return'.
		bool IsFirstReturn () const
			{ return (GetReturnNumber() == 1); }
			
		//! Determine if point has 'key point' attribute.
		bool IsKeyPoint () const;
			
		//! Determine if point is 'last return'.
		bool IsLastReturn () const
			{ return (GetReturnNumber() == GetReturnCount()); }

		//! Determine if scan sweep is in positive direction.
		bool IsScanPositive () const;
			
		//! Determine if point has 'synthetic' attribute set.
		bool IsSynthetic () const;

		//! Set classification flag bits
		void SetClassFlags (
			UINT8 classflags						//!< Flag bits 
			);
			
		//! Set classification.
		void SetClassification (
			UINT8 classification					//!< Classification value (range depends on format)
			);

		//! Set classification.
		void SetClassification (
			UINT8 classification,				//!< Classification value (range depends on format)
			bool IsSynthetic,
			bool IsKeyPoint,
			bool IsDeleted
			);

		void SetColor (
			const COLOR& color
			);

		//! Set 'deleted' status.
		void SetDeleted (
			bool deleted							//!< Whether marked as 'deleted' or not
			);

		void SetIntensity (
			UINT16 intensity
			) { *reinterpret_cast<UINT16*>(GetData() + OFFSET_Intensity) = intensity; }

		//! Set 'key point' status.
		void SetKeyPoint (
			bool keypoint							//!< Whether key point or not
			);

		void SetReturnValues (
			UINT8 ReturnNumber,
			UINT8 ReturnCount
			);

		void SetScanAngle (
			float ScanAngle
			);

		void SetScanFlags (
			bool IsScanPositive,
			bool IsEndOfSweep
			);

		void SetSourceID (
			UINT16 SourceID
			) { *reinterpret_cast<UINT16*>(GetData() + GetOffsetSourceID()) = SourceID; }

		//! Set 'synthetic' status.
		void SetSynthetic (
			bool synthetic							//!< Whether synthetic or not
			);

		void SetTime (
			double time
			);

		//! Set 'user data' value.
		void SetUserData (
			UINT8 userdata
			) { GetData()[OFFSET_UserData] = userdata; }

		#ifdef BYTEORDER_HiLo
		void SwapBytes (UINT8 *data) const;
		#endif

		ERRVALUE Write (FILE_RAW& file, INT64 offset) const;

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT

		static const int OFFSET_X =					0;		// INT32
		static const int OFFSET_Y =					4;		// INT32
		static const int OFFSET_Z =					8;		// INT32
		static const int OFFSET_Intensity =			12;	// UINT16
		static const int OFFSET_ReturnValues =		14;	// UINT8 bit field
		static const int OFFSET_UserData =			17;	// UINT8

		UINT8 GetFormat () const
			{ return (static_cast<const POINTARRAY&>(GetArray()).GetFormat()); }
		UINT16 GetOffsetColor () const
			{ return (static_cast<const POINTARRAY&>(GetArray()).GetOffsetColor()); }
		UINT16 GetOffsetSourceID () const
			{ return (static_cast<const POINTARRAY&>(GetArray()).GetOffsetSourceID()); }
		UINT16 GetOffsetTime () const
			{ return (static_cast<const POINTARRAY&>(GetArray()).GetOffsetTime()); }
		UINT16 GetOffsetWave () const
			{ return (static_cast<const POINTARRAY&>(GetArray()).GetOffsetWave()); }
		const UINT8* GetPtrWave () const
			{ return (GetData() + GetOffsetWave()); }

		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(POINTRECORD::CLASSFLAGS)
#endif

//=====================================================================================================================

//! Delegate for selecting point record.
typedef fastdelegate::FastDelegate<bool(const LAS::POINTRECORD&)> DELEGATE_BOOL_POINTRECORD;

//=====================================================================================================================

}	// End namespace LAS
}	// End namespace LIDAR

#undef	CLASSLIBEXPORT

#endif	// INC_LIDAR_POINTRECORD_H
