/**
 * \file matrix.h <mi32/matrix.h>
 * \brief Definitions for matrix manipulation functions
 *
 * \if NODOC
 * $Id: matrix.h_v 1.27 2007/03/06 14:13:21 mju Exp $
 *
 * $Log: matrix.h_v $
 * Revision 1.27  2007/03/06 14:13:21  mju
 * Remove unused prototypes.
 *
 * Revision 1.26  2007/03/05 21:29:24  mju
 * Remove unused prototypes.
 *
 * Revision 1.25  2007/02/02 14:40:35  mju
 * Disable deprecation warnings within header.
 *
 * Revision 1.24  2006/11/10 23:45:07  dfriberg
 * Deprecate this structure
 *
 * Revision 1.23  2005/03/31 16:57:15  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.22  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.21  2002/06/12 13:13:25  mju
 * nc.
 *
 * Revision 1.20  2000/12/22 17:50:23  dwilliss
 * Principle Componants function now takes real pointer for type-safety
 * instead of (void*)
 *
 * Revision 1.19  2000/06/20 17:56:01  sparsons
 * Genitor documentation.
 *
 * Revision 1.18  2000/06/14 13:26:27  mju
 * Include stddefns so compiles in isolation.
 *
 * \endif
**/

#ifndef NO_DEPRECATED

#ifndef	INC_MI32_MATRIX_H
#define	INC_MI32_MATRIX_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#if defined(__cplusplus)
extern "C" {
#endif


DEPRECATED typedef struct _MATRIX {
	double **matrix;							//!<  Matrix data 
	UINT16 rows;								//!<  Number of rows 
	UINT16 cols;								//!<  Number of columns 
	UINT32 flags;
	} *MATRIX;

#pragma warning (disable:4996)

#define	MATFLAG_Symmetric			0x00000001			//!<  Matrix is symmetric 
#define	MATFLAG_NoDiagonal		0x00000002			//!<  Don't allocate space for diagonal (symmetric only) 

//! Retrieve value from matrix.
#define	MatGetValue(m,i,j)		((m)->matrix[i][j])

//! Set single matrix entry to a specified value.
#define	MatSetValue(m,i,j,v)		((m)->matrix[i][j]=(v))

//! Determine if "symmetric" flag set for a matrix.
#define	MatIsSymmetric(m)			((m)->flags&MATFLAG_Symmetric)

//! Add one matrix to another of the same size.
int MatAddMatrix (
	MATRIX outmatrix, 						//!< Output matrix, may be the same as either input matrix
	MATRIX matrixA, 							//!< First input matrix
	MATRIX matrixB								//!< Second input matrix, if NULL will use outmatrix
	);

//! Copy one matrix to another.
//!
//! The output matrix will be resized if it exists and is not the same size as the input matrix.
int MatCopy (
	MATRIX *outmatrix, 						//!< Matrix to copy to, will be created if NULL
	MATRIX inmatrix							//!< Matrix to copy from
	);

//! Create a new matrix.
//!
//! Flags:
//!		MATFLAG_Symmetric		Matrix is symmetric, don't allocate space for elements above diagonal
//!		MATFLAG_NoDiagonal	Don't allocate space for diagonal elements (symmetric only)
//!
//!	All matrix elements will be initialized to 0.
int MatCreate (
	MATRIX *matrix, 							//!< Matrix returned
	UINT16 rows, 								//!< Number of rows in matrix
	UINT16 cols, 								//!< Number of columns in matrix
	UINT32 flags								//!< Flags
	);

//! Destroy a matrix.
void MatDestroy (
	MATRIX matrix								//!< Matrix to be destroyed
	);

//! Compute determinate of a square matrix.
int MatDeterminant (
	MATRIX matrix, 							//!< Matrix to compute determinant for
	double *detvalue							//!< Determinant returned
	);

//! Compute eigenvectors and eigenvalues of the SYMMETRIC matrix.
//!
//! WARNING: This function computes eigen-stuff for SYMMETRIC matrices ONLY.
//!	Example: correlation /covariation matrix. 
//!	NO CHECKING IS PERFORMED ON INPUT TO MAKE SURE IT'S SYMMETRIC.
int MatEigenvectors (
	MATRIX matrix,								//!< Matrix to use
	MATRIX *vectors,							//!< Matrix returned, that contain eigenvectors as columns
	MATRIX *values								//!< Matrix returned (vector), that contains eigenvalues
	);

//! Retrieve value from matrix with range check.
//!
//! @return Value in matrix. 
//! This function will return 0.0 for values outside the matrix.  It also handles symmetric matrices.
double MatGetValueTest (
	MATRIX matrix, 							//!< Matrix to retrieve value from
	UINT16 row, 								//!< Row in matrix (0 - numrows-1)
	UINT16 col									//!< Column in matrix (0 - numcols-1)
	);

//! Compute inverse of a square matrix.
int MatInverse (
	MATRIX outmatrix, 						//!< Inverse matrix computed
	MATRIX inmatrix, 							//!< Matrix to compute inverse of
	int maxiterations, 						//!< Maximum iterations for convergence (0 for default)
	double maxerror							//!< Maximum error, (0.0 for default)
	);

//! Multiply one matrix by another.
int MatMultMatrix (
	MATRIX outmatrix, 						//!< Output matrix, may be the same as either input matrix
	MATRIX matrixA, 							//!< Left matrix, will use outmatrix if NULL
	MATRIX matrixB								//!< Right matrix, will use outmatrix if NULL
	);

//! Multiply a matrix by a scalar.
void MatMultScalar (
	MATRIX matrix, 							//!< Matrix to use
	double value								//!< Value to multiply all matrix elements by
	);

//! Compute pseudoinverse of the matrix using SVD  
//!
//! If Input matrix is not singular pseudoinverse is equal to
//!	normal inverse of the matrix.	
//! EXPLANATION:
//!	Pseudoinverse is such matrix X that satisfy the following condition:
//!		A * X * A = A, if matrix A is singular X still exists.
int MatPseudoInverse (
	MATRIX inverse, 							//!< Output pseudoinverse
	MATRIX input, 								//!< Input matrix
	double tolerance							//!< Tolerance to use, if in doubt pass 0.0 and function will use built-in defaults
	);

//! Set all entries in a matrix to a specified scalar value.
void MatSetScalar (
	MATRIX matrix, 							//!< Matrix to use
	double value								//!< Value to set all matrix elements to
	);

//! Compute Singular Value Decomposition (SVD).
//!
//! This function computes SVD as A = U * W * V ;
//!	Output matrix U has the same dimension as input matrix A. 
//!	The diagonal matrix of singular values W is output as a vector.
//!	The matrix V (not transpose of V) is output as a matrix.
int MatSVD (
	MATRIX A,									//!< Input matrix
	MATRIX *U,									//!< Returned
	MATRIX *W,									//!< Returned
	MATRIX *V									//!< Returned
	);

//! Stable Solution of system of linear equation using SVD algorithm.
//!
//! A * X = B where A(M x N) matrix and X (N) vector. B(M) is also a vector.
//!	X is the output solution vector.
//!	Matrix A is decomposed into U(M x N), W(N) and V(N x N) matrices using SVD operation.
int MatSVDSolve (
	MATRIX A, 									//!< Input matrix of equations
	MATRIX B, 									//!< Input vector that contains right side of equations
													//! NOTE: you can pass ANY type of vector: numrows = 1 or numcols = 1
	MATRIX X,									//!< Solution returned
	double tolerance							//!< Tolerance to use, if in doubt pass 0.0 for defaults
	);

//! 
int MatSVDSolveAutoTolerance (
	MATRIX A, 
	MATRIX B, 
	MATRIX X										//!< Size of A->cols x 1
	);

//! 
int MatSVDSolveAutoToleranceFast (
	MATRIX A, 
	MATRIX U, 
	MATRIX W, 
	MATRIX V, 
	MATRIX B, 
	MATRIX X										//!< Size of A->cols x 1
	);

//! Subtract one matrix from another.
int MatSubMatrix (
	MATRIX outmatrix, 						//!< Output matrix, may be the same as either input matrix
	MATRIX matrixA, 							//!< Matrix to subtract FROM
	MATRIX matrixB								//!< Second to subtract, if NULL will subtract matrixA from outmatrix
	);

//! Compute trace of the matrix: sum of it's diagonal elements.
double MatTrace (
	MATRIX matrix								//!< Matrix to use
	);

//! Transpose a matrix.
int MatTranspose (
	MATRIX outmatrix, 						//!< Output matrix (may be the same as the input matrix)
	MATRIX inmatrix							//!< Input matrix
	);

#pragma warning (default:4996)

#if defined(__cplusplus)
}
#endif

#endif	//  #ifdef INC_MI32_MATRIX_H 
#endif	//  #ifdef NO_DEPRECATED
