/******************************************************************************
* \file mi32/rpcmodel.h
* \brief RPCMODEL class definitions.
* 
* \if NODOC
* $Id: rpcmodel.h_v 1.16 2013/04/29 16:47:23 mju Exp $
* $Log: rpcmodel.h_v $
* Revision 1.16  2013/04/29 16:47:23  mju
* Remove unused computeControlPoints and getDerivatives methods.
*
* Revision 1.15  2011/11/04 13:22:27  mju
* Remove redundant hdr.
*
* Revision 1.14  2010/11/29 18:21:27  mju
* Add interpolation enum.
*
* Revision 1.13  2009/07/07 20:56:59  mju
* Remove unused/unimplemneted methods.
* Use dpoint2d/3d as parms so passed more efficiently.
*
* Revision 1.12  2008/10/09 22:44:27  bcolombe
* Added ALOSPrism parsing
*
* Revision 1.11  2007/08/20 15:20:08  bcolombe
* Removed illegal qualifier
*
* Revision 1.8  2007/07/12 16:03:49  vdronov
* changed Resample argument list
*
* Revision 1.6  2003/11/25 18:14:29  vdronov
* added extra ComputeControlPoints method
*
* Revision 1.1  2003/07/18 19:29:11  vdronov
* Initial revision
*
* \endif
*******************************************************************************/

#ifndef	INC_MI32_RPCMODEL_H
#define	INC_MI32_RPCMODEL_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_CTRLPT_H
#include <mi32/ctrlpt.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class BITSET;
class FILEPATH;
class TRANS2D_MAPGEN;
namespace RVC {
	class OBJITEM;
	class RASTER;
	class GEOREFERENCE;
	}
#endif

class RPCMODEL {
	public:

		enum INTERPOLATION {
			INTERPOLATION_NearestNeighbor =		0,
			INTERPOLATION_Bilinear,
			INTERPOLATION_Bicubic
			};

		//! Default Constructor
		RPCMODEL (
			);

		//! Copy Constructor
		RPCMODEL (
			const RPCMODEL& model
			);

		//! Assignment operator
		RPCMODEL& operator= (
			const RPCMODEL& model
			);

		//! Clear object
		void Clear (
			) { m_Init = false; }

		//! Clear enhancement
		void ClearEnhancement (
			) { m_Enhancement = ENHANCEMENT_None; }

		//! Convert (latitude, longitude, height) to (row, column) in unrectifyed raster
		//! @return true if succesful convertion
		bool Convert (
			const DPOINT3D& ptGeogH,						//!< longitude,latitude,height
			DPOINT2D& ptImage									//!< Image coordinates returned
			) const;

		//! Enhance RPC Model with georeference's control points
		//! @return true if enhanced
		bool Enhance (
			const RVC::GEOREFERENCE& ObjGeoref
			);

		//! Enhance RPC Model with control points
		//! @return true if enhanced
		bool Enhance (
			const SIMPLE_ARRAY<CTRLPOINT3>& ControlPoints,
			const TRANS2D_MAPGEN& Transformation,
			const BITSET& Selection
			);

		//! Get RPC Model as string in original format
		//! @return reference of string containing RPC Model
		const MISTRING& GetRPCModelString (
			) const { return m_RPCModelString; };

		//! Init object with RPC Model in text file
		//! @return true if init
		bool Init (
			const FILEPATH& filepath
			);

		//! Init object with RPC Model in text subobject
		//! @return true if init
		bool Init (
			const RVC::OBJITEM& objitem		//!<	TEXT OBJECT containing RPC Model text(IKONOS or QuickBird)
			);

		//! Init object with RPC Model in text subobject of given georeference object.
		//! @return true if init
		bool Init (
			const RVC::GEOREFERENCE& ObjGeoref
			);

		//! Is object was initiated
		//! @return true if initiated
		bool IsInit () const { return m_Init; };

		//! Resample raster
		//! @return value greater or equal 0 if succesful resampling
		ERRVALUE Resample (
			const RVC::RASTER& ObjRasterSrc,
			const RVC::RASTER& ObjRasterElev,
			const INTERPOLATION interpolation,
			const TRANS2D_MAPGEN& TransTgtToMap,
			RVC::RASTER& ObjRasterTgt
			) const;

		//! Resample raster
		//! @return value greater or equal if succesful convertion
		ERRVALUE Resample (
			const RVC::RASTER& ObjRasterSrc,
			const RVC::GEOREFERENCE& ObjGeorefSrc,
			const RVC::RASTER& ObjRasterElev,
			const INTERPOLATION interpolation,
			const TRANS2D_MAPGEN& TransTgtToMap,
			RVC::RASTER& ObjRasterTgt
			) const;

		//! Set geoid height
		void SetGeoidHeight (
			const double height
			) { m_GeoidHeight = height; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum ENHANCEMENT {
			ENHANCEMENT_None,
			ENHANCEMENT_Constant,
			ENHANCEMENT_Linear,
			ENHANCEMENT_Affine,
			ENHANCEMENT_LeastSquares
			};

		bool m_Init;

		double m_RowOffset;
		double m_ColumnOffset;
		double m_LatitudeOffset;
		double m_LongitudeOffset;
		double m_HeightOffset;

		double m_RowScale;
		double m_ColumnScale;
		double m_LatitudeScale;
		double m_LongitudeScale;
		double m_HeightScale;

		double m_RowNum[20];
		double m_RowDen[20];

		double m_ColumnNum[20];
		double m_ColumnDen[20];

		ENHANCEMENT m_Enhancement;
		double m_A[3];
		double m_B[3];

		double m_GeoidHeight;
		MISTRING m_RPCModelString;

		void Copy (const RPCMODEL& rhs);

		bool Parse (const MISTRING& string);
		ERRVALUE ParseALOSPrism (const MISTRING& string);
		ERRVALUE ParseIKONOS (const MISTRING& string);
		ERRVALUE ParseQuickBird (const MISTRING& string);

		ERRVALUE ReadRPCModel (const FILEPATH& filepath);
		ERRVALUE ReadRPCModel (const RVC::OBJITEM& objitem);

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#endif	// INC_MI32_RPCMODEL_H 
