/**
 * \file mi32/thinplatespline.h
 * \brief THINPLATESPLINE definition
 *
 * \if NODOC
 * $Id: thinplatespline.h_v 1.3 2012/08/17 21:15:47 mju Exp $
 *
 * $Log: thinplatespline.h_v $
 * Revision 1.3  2012/08/17 21:15:47  mju
 * Export from DLL.
 *
 * Revision 1.2  2012/07/30 20:34:10  mju
 * Add docs.
 *
 * Revision 1.1  2012/07/30 20:04:41  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_THINPLATESPLINE_H
#define	INC_MI32_THINPLATESPLINE_H

#ifndef	INC_MI32_MIMATRIX_H
#include <mi32/mimatrix.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifdef GEOMDLL
	#define CLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSEXPORT MI_DLLCLASSIMPORT
#endif


//! Thin Plate Spline (TPS) interpolation method using a "minimally bended" smooth surface that passes through all given points.
//! The name "Thin Plate" comes from the fact that a TPS more or less simulates how a thin metal plate would behave if it was forced
//! through the same control points.  A TPS of 3 control points is a plane, more than 3 is generally a curved surface, less than 3 is undefined.
//!
//! TPS's are commonly used in representing shape transformations, for example image morphing, and shape detection/matching.
//!
//! Consider a set of source and target 2D points, S and T.  Let Z(i) = Tx(i) - Sx(i).  Then fit a TPS over points (Sx(i), Sy(i), Z(i)) to
//! get an interpolation function for translating points in the X direction.  Repeat the same for Y.  Note that such a transformation will
//! not in general be mathematically reversible
//!
//! In some cases, such as when the control point coordinates are noisy, you may want to relax the interpolation requirements slightly so
//! that the resulting surface doesn't have to go exactly through the control points.  This is called "regularization" and is controlled by
//! the regularization parameter (lambda).  If this value is zero, interpolation is exact, and as it approaches infinity, the resulting TPS
//! surface is reduced to a least-squares fitted plane ("bending energy" of a plane is 0).
class CLASSEXPORT THINPLATESPLINE {
	public:

		//! Constructor.
		THINPLATESPLINE ();

		//! Destructor.
		~THINPLATESPLINE ();

		//! Compute coefficients for evaluation.
		ERRVALUE Compute (
			const DPOINT3D *pts,							//!< Control points
			unsigned np,									//!< Number of points
			double regularization						//!< Regularization factor, 0 for exact fit, infinite for least-square fitted plane
			);

		//! Evaluate at specified coordinates.
		double Evaluate (
			const DPOINT2D& pt							//!< Coordinates to evaluate at.
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DOUBLE_ARRAY<DPOINT2D> m_pts;
		MIMATRIX m_MatC;
		double m_z0;
		double m_zx;
		double m_zy;
		#endif
	};

#undef CLASSEXPORT

#endif	// INC_MI32_THINPLATESPLINE_H
