/**
 * \file mi32/ucstring.h
 * \brief Definitions for Unicode string functions and related text conversion
 *
 * \if NODOC
 * $Id: ucstring.h_v 1.142 2008/11/17 18:05:21 mju Exp $
 *
 * $Log: ucstring.h_v $
 * Revision 1.142  2008/11/17 18:05:21  mju
 * Need stdio hdr for non-windows to get size_t defined.
 *
 * Revision 1.141  2008/11/17 14:00:25  mju
 * Move ...isdigit to ucchar hdr.
 *
 * Revision 1.140  2008/11/14 15:32:47  mju
 * Move non-string fns to mucdefns, ucchar or ucdata hdrs.
 *
 * Revision 1.139  2008/09/08 13:32:16  mju
 * Remove extraneous comment.
 *
 * Revision 1.136  2006/10/23 15:04:51  dwilliss
 * Added ucstrtok2, which is thread-safe
 *
 * Revision 1.135  2006/10/10 17:49:13  dwilliss
 * Added CP932
 *
 * Revision 1.132  2006/05/03 19:47:03  scowan
 * Updated uc is space to return true on values 0x09 thru 0x0D and 0x20.
 * As per ANSI standards.
 *
 * Revision 1.117  2002/09/09 16:35:42  mju
 * \endif
**/

#ifndef INC_MI32_UCSTRING_H
#define INC_MI32_UCSTRING_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#if !defined(WIN32) && !defined(INC_STDIO_H)
#include <stdio.h>
#define INC_STDIO_H
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif

//!\addtogroup UCSTRING Unicode String Functions
//!@{

#if defined(__cplusplus)
extern "C" {
#endif

//! Convert an ASCII string to Unicode (with length limit).
//!
//! Note, this function doesn't do any character set mapping.  It
//! simply casts all the source characters to (MIUNICODE)
LIBEXPORT MIUNICODE* strntouc (
	MIUNICODE *dest, 
	const char *source, 
	int len
	);

//! Convert an ASCII string to Unicode.
//!
//! Note, this function doesn't do any character set mapping.  It
//! simply casts all the source characters to (MIUNICODE)
LIBEXPORT MIUNICODE* strtouc (
	MIUNICODE *dest,	
	const char *source
	);

//! Convert an ASCII string to Unicode and append to existing string.
//!
//! Note, this function doesn't do any character set mapping.  It
//! simply casts all the source characters to (MIUNICODE)
LIBEXPORT MIUNICODE* strtouccat (
	MIUNICODE *dest, 
	const char * source
	);

//! MIUNICODE version of strdup().
//!
//! Note, this function doesn't do any character set mapping.  It
//! simply casts all the source characters to (MIUNICODE).
//! The resulting string should be MmFree()'d.
LIBEXPORT MIUNICODE* strtoucdup (
	const char*
	);

//! MIUNICODE version of strcat().
LIBEXPORT MIUNICODE* ucstrcat (
	MIUNICODE *dest, 
	const MIUNICODE *source
	);

//! MIUNICODE version of strchr().
//!
//! Find the first occurance of a given character in a Unicode string.
//! Returns a pointer to that character or NULL if not found.
LIBEXPORT MIUNICODE* ucstrchr (
	const MIUNICODE *p, 
	MIUNICODE value
	);

//! Compare two MIUNICODE strings.
//!
//! Returns -1 if (p1 < p2),
//! 			1 if (p1 > p2)
//! 			0 if (p1 == p2)
LIBEXPORT int ucstrcmp (
	const MIUNICODE *p1, 
	const MIUNICODE *p2
	);

//! Unicode version of strcpy().
LIBEXPORT MIUNICODE* ucstrcpy (
	MIUNICODE *dest, 
	const MIUNICODE *source
	);

//! Unicode version of strdup().
LIBEXPORT MIUNICODE* ucstrdup (
	const MIUNICODE *s
	);

//! Unicode version of strlen().
LIBEXPORT size_t ucstrlen (
	const MIUNICODE *p
	);

//! Compare two MIUNICODE strings (case insensitive).
//!
//! Returns -1 if (p1 < p2),
//! 			1 if (p1 > p2)
//! 			0 if (p1 == p2)
LIBEXPORT int ucstricmp (
	const MIUNICODE *p1, 
	const MIUNICODE *p2
	);

//! Convert MIUNICODE string to lowercase (in-place).
//!
//! Modifies the input string
LIBEXPORT MIUNICODE* ucstrlwr (MIUNICODE *p);

//! MIUNICODE version of strncat().
LIBEXPORT MIUNICODE* ucstrncat (
	MIUNICODE *p1, 
	const MIUNICODE *p2, 
	int len
	);

//! Compare two MIUNICODE strings (length limited).
//!
//! Returns -1 if (p1 < p2),
//! 			1 if (p1 > p2)
//! 			0 if (p1 == p2)
LIBEXPORT int ucstrncmp (
	const MIUNICODE *p1, 
	const MIUNICODE *p2, 
	int len
	);

//! MIUNICODE version of strncpy().
LIBEXPORT MIUNICODE* ucstrncpy (
	MIUNICODE *dest, 
	const MIUNICODE *source, 
	int len
	);

//! Compare two MIUNICODE strings (length limited and case insensitive).
//!
//! Returns -1 if (p1 < p2),
//! 			1 if (p1 > p2)
//! 			0 if (p1 == p2)
LIBEXPORT int ucstrnicmp (
	const MIUNICODE *p1, 
	const MIUNICODE *p2, 
	int len
	);

//! MIUNICODE version of strpbrk().
//!
//! Locate the first occurrence in the string s of any of the characters in the 
//! string accept.  Returns a pointer to it or NULL if not found.
LIBEXPORT MIUNICODE* ucstrpbrk (
	MIUNICODE *s, 
	MIUNICODE *accept
	);

//! MIUNICODE version of strrchr().
//!
//! Locates the last occurrence of the character value in the string s.
//! Returns a pointer to it or NULL if not found.
LIBEXPORT MIUNICODE* ucstrrchr (
	MIUNICODE *s, 
	MIUNICODE value
	);

//! MIUNICODE version of strspn().
//!
//! Returns the length of the initial segment of s which consists entirely of 
//! characters in accept.
LIBEXPORT size_t ucstrspn (
	const MIUNICODE *s, 
	const MIUNICODE *accept
	);

//! MIUNICODE version of strstr().
//!
//! Finds the first occurrence of the substring p in the string s.
//! Returns a pointer to it or NULL if not found.
LIBEXPORT MIUNICODE* ucstrstr (
	MIUNICODE *s, 
	const MIUNICODE *p
	);

//! Extract token from string
//!
//! A "token" is a nonempty string of characters not occurring in the string
//! delim, followed by \\0 or by a character occurring in delim.
//!
//! The ucstrtok() function can be used to parse the string s
//! into tokens.  The first call the ucstrtok() should have s
//! as its first argument.  Subsequent calls should have the
//! first argument set to NULL.  Each call returns a pointer to
//! next token, or ULL when no more tokens are found.
//!
//! If a token ends with a delimiter, this delimiting character
//! is saved for the next call to ucstrtok().  The delimiter string
//! may be different for each call.
//!
//! <b>Warning:</b> This function is \e not reentrant and it modifies
//! the string passed as the first parameter.  
LIBEXPORT MIUNICODE* ucstrtok (
	MIUNICODE *s, 
	MIUNICODE *delim
	);

//! Extract token from string
//!
//! A "token" is a nonempty string of characters not occurring in the string
//! delim, followed by \\0 or by a character occurring in delim.
//!
//! The ucstrtok2() function can be used to parse the string s
//! into tokens.  The first call the ucstrtok2() should have s
//! as its first argument.  Subsequent calls should have the
//! first argument set to NULL.  Each call returns a pointer to
//! next token, or ULL when no more tokens are found.
//!
//! If a token ends with a delimiter, this delimiting character
//! is saved for the next call to ucstrtok2().  The delimiter string
//! may be different for each call.
//!
//! The last parameter is a reference to an MIUNICODE* that ucstrtok2
//! uses to keep track of the next starting position.  This allows it
//! to avoid the use of a static variable and makes it thread-safe.
//!
//! <b>Warning:</b> This function \e modifies
//! the string passed as the first parameter.  
LIBEXPORT MIUNICODE* ucstrtok2 (
	MIUNICODE *s, 
	MIUNICODE *delim,
	MIUNICODE *& state
	);

//! Convert MIUNICODE to uppercase (in-place).
LIBEXPORT MIUNICODE* ucstrupr (
	MIUNICODE *p
	);

//! Convert MIUNICODE string to ASCII.
//! \deprecated, Use appropriate MISTRING or FILEPATH method instead.
//! Note, this just strips the high byte, so any characters > 255 will
//! produce garbage in the output 
LIBEXPORT DEPRECATED char* uctostr (
	char *dest, 
	const MIUNICODE *source
	);

//! Convert MIUNICODE string to ASCII (with length-limit).
//! \deprecated, Use appropriate MISTRING or FILEPATH method instead.
//! Note, this just strips the high byte, so any characters > 255 will
//! produce garbage in the output 
LIBEXPORT DEPRECATED char* ucntostr (
	char *dest, 
	const MIUNICODE *source, 
	int len
	);

//! Return an ASCII copy of a MIUNICODE string.
//! \deprecated, Use appropriate MISTRING or FILEPATH method instead.
//! Note, this just strips the high byte, so any characters > 255 will
//! produce garbage in the output 
LIBEXPORT DEPRECATED char* uctostrdup (
	const MIUNICODE*
	);

#if defined(__cplusplus)
}
#endif


//!@} 

#undef LIBEXPORT 

#endif  // INC_MI32_UCSTRING_H
