/**
 * \file rvc/cameracb.h 
 * \brief interface for the RVC::CAMERACALIB class.
 *
 * \if NODOC
 * $Id: cameracb.h_v 1.12 2007/09/06 17:18:14 scowan Exp $
 *
 * $Log: cameracb.h_v $
 * Revision 1.12  2007/09/06 17:18:14  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.11  2005/12/09 17:29:14  scowan
 * Updated to use credentials.
 *
 * Revision 1.10  2005/07/19 20:53:40  scowan
 * Fixed virtual method issue.
 *
 * Revision 1.9  2005/01/19 23:21:20  scowan
 * Support of descriptor class.
 *
 * Revision 1.8  2003/10/07 17:35:46  scowan
 * Object virtual method changes.
 *
 * Revision 1.7  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.5  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2001/06/15 19:04:06  scowan
 * More const.
 *
 * Revision 1.3  2001/06/14 19:24:33  scowan
 * Added to RVC namespace.
 *
 * Revision 1.2  2000/10/19 21:13:25  scowan
 * Checked in for vacation.
 *
 * Revision 1.1  2000/09/29 19:31:55  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::CAMERACALIB rvc/cameracb.h
//!	\brief Manages RVC::OBJTYPE_CameraCalib objects.  
//!
//!	The interface currently wraps the MfCameraCalib functions.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_CameraCalib.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_CameraCalib.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Raster
//!
//!	Read(), Make() and Write() methods inherited from RVC::SIMPLE class.  View RVC::SIMPLE docs for inherited methods

#ifndef INC_RVC_CAMERACB_H
#define INC_RVC_CAMERACB_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_POINT_H
	#include <mi32/point.h>
#endif

namespace RVC {

class CAMERACALIB : public RVC::SIMPLE {
	public:

		CAMERACALIB (
			);

		CAMERACALIB (		   					//! Copy Constructor
			const RVC::CAMERACALIB& rhs					
			);

		virtual ~CAMERACALIB (
			);

		RVC::CAMERACALIB& operator= (					//! Assignment operator
			const RVC::CAMERACALIB& rhs
			);

		//! Get camera position
		//! @return Camera position in 3D coordinates
		const DPOINT3D& GetCameraPosition (
			) const;

		//! Get Epipolar angle
		//! @return epipolar angle
		double GetEpipolarAngle (
			) const;

		//! Get focal length
		//! @return focal length
		double GetFocalLength (
			) const;

		//! Get image scale
		//! @return image scale
		double GetImageScale (
			) const;

		//! Get principal point
		//! @return principal point
		const DPOINT2D& GetPrincPoint (
			) const;

		//! Get swing angle
		//! @return swing angle
		double GetSwingAngle (
			) const;

		//! Get tilt angle
		//! @return tilt angle
		double GetTiltAngle (
			) const;
		
		//! Set camera position
		void SetCameraPosition (
			const DPOINT3D& CameraPosition			//!< New camera position
			);

		//! Set Epipolar Angle
		void SetEpipolarAngle (
			double EpipolarAngle					//!< New epipolar angle
			);

		//! Set focal length
		void SetFocalLength (
			double FocalLength						//!< New focal length
			);

		//! Set image scale
		void SetImageScale (
			double ImageScale						//!< New image scale
			);

		//! Set principal point
		void SetPrincPoint (
			const DPOINT2D& PrincPoint				//!< New principal point
			);

		//! Set swing angle
		void SetSwingAngle (
			double SwingAngle						//!< New swing angle
			);

		//! Set tilt angle
		void SetTiltAngle (
			double TiltAngle						//!< New tilt angle
			);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		DPOINT2D m_PrincPoint;					//!< Principle point
		DPOINT3D m_CameraPosition;				//!< Position of camera relative to raster
		double m_EpipolarAngle;					//!< Angle to epipolar line for stereo analysis
		double m_TiltAngle;						//!< Angle measured in principle plane from optical axis of camera
		double m_SwingAngle;					//!< Direction of tilt with respect to photographic axes
		double m_FocalLength;					//!< Focal length of camera converted to cells
		double m_ImageScale;					//!< Size of pixel in camera image plane

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_CAMERACB_H
