/**
 * \file rvc/tinsurface.h
 * \brief RVC::OP::TINSURFACE definitions
 *
 * \if NODOC
 * $Id: optinsurface.h_v 1.1 2013/01/10 18:31:03 mju Exp $
 *
 * $Log: optinsurface.h_v $
 * Revision 1.1  2013/01/10 18:31:03  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_TINSURFACE_H
#define	INC_RVC_TINSURFACE_H

#ifndef	INC_RVC_TIN_H
#include <rvc/tin.h>
#endif

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Allows calculation of surface properties from TIN.
class TINSURFACE {
	public:

		//! Id used to specify element within TINSURFACE context.
		//! This allows a single value to uniquely specify any type of element within a TIN object, as well as
		//! certain non-element values.
		typedef UINT32 ELEMID;

		static const ELEMID ELEMID_NoElement = UINT32_MAX;			//! No element exists
		static const ELEMID ELEMID_NotSpecified = UINT32_MAX-1;	//! No element specified

		//! Array of element IDs.
		typedef SIMPLE_ARRAY<ELEMID> ELEMID_ARRAY;

		//! Constructor.
		TINSURFACE ();

		//! Destructor.
		~TINSURFACE ();

		//! Close objects.
		void Close ();

		//! Get ELEMENT from ELEMID.
		void GetElement (
			ELEMID elemid,									//!< Element ID
			RVC::ELEMENT& elem							//!< Element returned
			) const;

		//! Get ELEMID for specified ELEMENT.
		ELEMID GetElementID (
			const RVC::ELEMENT& elem					//!< Element
			) const;

		//! Interpolate value at specified point on surface.
		//! @return TRUE(1) if value could be interpolated, FALSE(0) if not, < 0 if error
		int Interpolate (
			const DPOINT2D& pt,							//!< Point in TIN coordinates
			double& z,										//!< Interpolated value returned
			ELEMID elemid = ELEMID_NotSpecified		//!< Element ID for interpolation, if known
			) const;

		//! Interpolate values for POLYLINE.
		//! Resulting 3D POLYLINE will contain points at input locations as well as wherever input segments intersect
		//! edges in the TIN.
		ERRVALUE Interpolate (
			const POLYLINE& input,						//!< Line in TIN coordinates
			POLYLINE& result,								//!< Resulting line returned
			double maxspacing = 0						//!< Maximum horizontal spacing between consecutive points in result, 0 for no maximum
			) const;

		//! Determine if object is open.
		bool IsOpen () const
			{ return (m_ObjTIN.IsOpen()); }

		//! Open object.
		ERRVALUE Open (
			const RVC::OBJITEM& ObjItem				//!< Object location
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		TIN m_ObjTIN;

		TINSURFACE (const TINSURFACE&);
		TINSURFACE& operator= (const TINSURFACE&);
		#endif
	};

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_TINSURFACE_H
