/**
 * \file rvc/otypeset.h 
 * \brief Define RVC::OBJTYPESET class.
 *
 * \if NODOC
 * $Id: otypeset.h_v 1.16 2012/07/27 14:09:45 mju Exp $
 *
 * $Log: otypeset.h_v $
 * Revision 1.16  2012/07/27 14:09:45  mju
 * Add containsOnly.
 *
 * Revision 1.15  2009/10/23 17:07:41  mju
 * Add addValidParentTypes, getValidParentTypeSet.
 *
 * Revision 1.14  2009/04/07 15:03:37  mju
 * Add getIniTag and implement in dll.
 *
 * Revision 1.13  2009/03/20 14:16:55  mju
 * Add ctor using 4 objtypes.
 *
 * Revision 1.11  2003/10/22 16:16:49  scowan
 * Added operator append for another obj type set.
 *
 * Revision 1.4  2001/06/14 21:52:25  mju
 * Add GetFirst/GetNext methods.
 *
 * Revision 1.1  2001/06/13 16:19:21  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OTYPESET_H
#define	INC_RVC_OTYPESET_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	INC_RVC_OBJTYPE_H
#include <rvc/objtype.h>
#endif

#ifndef	INC_STRING_H
#ifndef GENERATING_DOXYGEN_OUTPUT
#include <string.h>		// Includes defn for memset()
#define	INC_STRING_H
#endif
#endif

#ifdef RVCSYSDLL
	#define CLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace RVC {

//=====================================================================================================================
//! \class OBJTYPESET rvc/otypeset.h
//! \brief Class to represent "set" of RVC object types.
class CLASSEXPORT OBJTYPESET {
	public:

		//! Default constructor, creates empty set.
		OBJTYPESET (
			) { Clear(); }

		//! Implicit constructor from OBJTYPE.
		//! Allows a single OBJTYPE value to be passed wherever OBJTYPESET is specified.
		OBJTYPESET (
			RVC::OBJTYPE objtype
			) { Clear(); operator+=(objtype); }

		//! Convenience constructor for specifying two object types.
		OBJTYPESET (
			RVC::OBJTYPE objtype1,
			RVC::OBJTYPE objtype2
			) { Clear(); operator+=(objtype1); operator+=(objtype2); }

		//! Convenience constructor for specifying three object types.
		OBJTYPESET (
			RVC::OBJTYPE objtype1,
			RVC::OBJTYPE objtype2,
			RVC::OBJTYPE objtype3
			) { Clear(); operator+=(objtype1); operator+=(objtype2); operator+=(objtype3); }

		//! Convenience constructor for specifying four object types.
		OBJTYPESET (
			RVC::OBJTYPE objtype1,
			RVC::OBJTYPE objtype2,
			RVC::OBJTYPE objtype3,
			RVC::OBJTYPE objtype4
			) { Clear(); operator+=(objtype1); operator+=(objtype2); operator+=(objtype3); operator+=(objtype4); }

		//! Add type to set.
		OBJTYPESET& operator+= (
			RVC::OBJTYPE rhs
			) { if (rhs == OBJTYPE_All) SetAll(); else m_objtypebits[rhs/32] |= 1 << (rhs % 32); return (*this); }

		//! Add type set to set.
		OBJTYPESET& operator+= (
			const RVC::OBJTYPESET& rhs
			) { for (INT32 i = 0; (i < (OBJTYPE_LAST+31)/32); ++i) m_objtypebits[i] |= rhs.m_objtypebits[i]; return (*this); }

		//! Remove type from set.
		OBJTYPESET& operator-= (
			RVC::OBJTYPE rhs
			) { if (rhs == OBJTYPE_All) Clear(); else { m_objtypebits[0] &= ~1; m_objtypebits[rhs/32] &= ~(1 << (rhs % 32)); } return (*this); }

		//! Add type to set.
		void AddType (
			RVC::OBJTYPE objtype			//! Type to add, must not be OBJTYPE_All or OBJTYPE_LAST.
			) { m_objtypebits[objtype/32] |= 1 << (objtype % 32); }

		//! Add valid parent types to set based on specified OBJTYPE.
		void AddValidParentTypes (
			RVC::OBJTYPE objtype
			);

		//! Clear set.
		void Clear (
			) { memset(m_objtypebits,0,sizeof(m_objtypebits)); return; }

		//! Determine if set contains specified object type.
		bool Contains (
			RVC::OBJTYPE objtype
			) const { return ((m_objtypebits[objtype/32] & (1 << (objtype % 32))) != 0); }

		//! Determine if set contains only specified object type.
		bool ContainsOnly (
			RVC::OBJTYPE objtype
			) const { return (GetFirst() == objtype && GetNext(objtype) == OBJTYPE_LAST); }

		//! Get first object type in set.
		//! @return First object type or OBJTYPE_LAST if no more in set.
		RVC::OBJTYPE GetFirst (
			) const { return (GetNext(OBJTYPE_All)); }

		//! Get Ini tag string for set.
		void GetIniTag (
			char *str			// String, must be large enought to hold at least 32 characters
			) const;

		//! Get set of valid parent types for any type in this set.
		void GetValidParentTypeSet (
			OBJTYPESET& ValidParentTypeSet,
			bool clear = true
			) const;

		//! Get next object type in set.
		RVC::OBJTYPE GetNext (
			OBJTYPE objtype
			) const {
			do {
				++objtype;
				if (Contains(objtype)) break;
				} while (objtype < OBJTYPE_LAST);
			return (objtype);
			}

		void SetAll (
			) { memset(m_objtypebits,255,sizeof(m_objtypebits)); return; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_objtypebits[(OBJTYPE_LAST+31)/32];
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End of RVC namespace

#undef CLASSEXPORT

#endif	// INC_RVC_OTYPESET_H
