/**
 * \file georeference.h <spatmod/georeference.h>
 * \brief Spatial Modelling (SPATMOD) GEOREFERENCE class definitions.
 *
 * \if NODOC
 *
 * $Id: georeference.h_v 1.13 2011/10/11 13:56:15 mju Exp $
 *
 * $Log: georeference.h_v $
 * Revision 1.13  2011/10/11 13:56:15  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.12  2011/10/05 20:49:15  mju
 * Use trans2daffine hdr.
 *
 * Revision 1.10  2011/01/13 15:22:30  mju
 * Add axisorientation an require in setCtrlPoints.
 *
 * Revision 1.9  2011/01/12 17:37:24  mju
 * Add model_invalid.
 *
 * Revision 1.8  2007/04/13 20:38:20  mju
 * Add implicit ctor from trans2d_mapgen.
 *
 * Revision 1.7  2006/11/30 20:26:56  mju
 * Add computeScale.
 *
 * Revision 1.5  2006/08/03 14:35:38  mju
 * Change to use applyTransInput and forward transformation.
 *
 * Revision 1.4  2006/02/02 15:58:23  mju
 * Add applyTransImage.
 *
 * Revision 1.2  2005/12/16 17:39:07  mju
 * Add getModel.
 *
 * Revision 1.1  2005/12/16 16:06:59  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_GEOREFERENCE_H
#define	INC_SPATMOD_GEOREFERENCE_H

#ifndef	INC_MI32_TRANS2DMAPGEN_H
#include <mi32/trans2dmapgen.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class TRANS2D_AFFINE;
#endif

namespace SPATMOD {
//=====================================================================================================================

//! Container for general georeference definition.
class GEOREFERENCE {
	public:

		//! Transformation model.
		// Values are chosen to match TRANS2DMODEL for convenience
		enum MODEL {
			MODEL_Invalid =				0,			//!< Invalid or undefined
			MODEL_Conformal =				0x0700,
			MODEL_Affine =					0x0100,
			MODEL_PlaneProjective =		0x0200,
			MODEL_Bilinear =				0x0300,
			MODEL_PolynomialOrder2 =	0x0402,
			MODEL_PolynomialOrder3 =	0x0403,
			MODEL_PolynomialOrder4 =	0x0404,
			MODEL_PiecewiseAffine =		0x8100,
			};

		//! Relative axis orientations.
		// Values are chosen to match TRANS2DMODEL for convenience
		enum AXISORIENTATION {
			AXISORIENTATION_Same =		0,			//!< Axes have same orientation, typical for geometric data
			AXISORIENTATION_Flip =		0x4000	//!< Axes are flipped relative to each other, typical for imagery
			};

		//! Default constructor.
		GEOREFERENCE ();

		//! Copy constructor.
		GEOREFERENCE (
			const GEOREFERENCE& rhs
			);

		//! Implicit constructor from TRANS2D_MAPGEN.
		//! Only the "input" side of the TRANS2D_MAPGEN instance is used.
		GEOREFERENCE (
			const TRANS2D_MAPGEN& Trans				//!< Transformation to new coordinates
			);

		//! Destructor.
		~GEOREFERENCE ();

		//! Assignment.
		GEOREFERENCE& operator= (
			const GEOREFERENCE& rhs
			);

		//! Apply transformation to "input" coordinates.
		ERRVALUE ApplyTransInput (
			const TRANS2D_AFFINE& Trans				//!< Transformation to new coordinates
			);

		//! Compute "scaling" from object to georeferenced units.
		//! Scaling returned will be to units of CRS X/Y axes or to linear meter.
		ERRVALUE ComputeScale (
			const DPOINT2D& point,						//!< Point in "object" coordinates
			DPOINT2D& scale,								//!< Scale returned
			bool ConvertToMeters							//!< Convert to linear meters if possible
			) const;

		//! Get relative axis orientation.
		AXISORIENTATION GetAxisOrientation () const;

		//! Get coordinate reference system.
		const SPATREF::COORDREFSYS& GetCRS (
			) const { return (m_TransGen.GetInputMapCoordRefSys()); }

		//! Get control point array.
		const DOUBLE_ARRAY<CTRLPOINT3>& GetCtrlPoints (
			) const { return (m_TransGen.GetInputCtrlPoints()); }

		//! Get tranformation model.
		MODEL GetModel (
			) const;

		//! Get affine transformation if possible.
		ERRVALUE GetTransAffine (
			TRANS2D_AFFINE& TransAffine,
			const DRECT2D& extent,
			double accuracy = 1.0
			) const;

		//! Get general transformation from source to defined CRS.
		const TRANS2D_MAPGEN& GetTransGen (
			) const { return (m_TransGen); }

		//! Determine if has control points.
		bool HasCtrlPoints (
			) const { return (GetCtrlPoints().GetNumItems() > 0); }

		//! Set control points and compute transformation.
		ERRVALUE SetCtrlPoints (
			const SPATREF::COORDREFSYS& CoordRefSys,		//!< Coordinate Reference System
			const DOUBLE_ARRAY<CTRLPOINT3>& CtrlPoints,	//!< Control points to set
			AXISORIENTATION AxisOrient,						//!< Axis orientation
			MODEL Model = MODEL_Affine							//!< Transformation model
			) { return (SetCtrlPoints(CoordRefSys,CtrlPoints,CtrlPoints.GetNumItems(),AxisOrient,Model)); }

		//! Set control points and compute transformation.
		ERRVALUE SetCtrlPoints (
			const SPATREF::COORDREFSYS& CoordRefSys,		//!< Coordinate Reference System
			const CTRLPOINT3 *CtrlPoints,						//!< Control points to set
			int NumCtrlPoints,									//!< Number of control points
			AXISORIENTATION AxisOrient,						//!< Axis orientation
			MODEL Model = MODEL_Affine							//!< Transformation model
			);

		//! Set affine transformation.
		//! Will clear any previously set control points.
		void SetTransAffine (
			const SPATREF::COORDREFSYS& CoordRefSys,
			const TRANS2D_AFFINE& TransAffine				//!< Transformation to set
			);

		//! Set general transformation.
		//! Will clear any previously set control points.
		//! Only the "input" side of the TRANS2D_MAPGEN instance is used.
		void SetTransGen (
			const TRANS2D_MAPGEN& TransGen					//!< Transformation to set
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		TRANS2D_MAPGEN m_TransGen;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_GEOREFERENCE_H
