/**
 * \file imagefilterlinear.h <spatmod/imagefilterlinear.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_LINEAR definition.
 *
 * \if NODOC
 * $Id: imagefilterlinear.h_v 1.3 2006/01/26 17:54:05 mju Exp $
 *
 * $Log: imagefilterlinear.h_v $
 * Revision 1.3  2006/01/26 17:54:05  mju
 * Switch tgt/src sample indices get/setCoefficient.
 *
 * Revision 1.2  2005/12/16 16:21:48  mju
 * Include doublear hdr.
 *
 * Revision 1.1  2005/08/19 14:25:51  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERLINEAR_H
#define	INC_SPATMOD_IMAGEFILTERLINEAR_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

class MIMATRIX;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to perform linear combination on samples in image.
class FILTER_LINEAR : public FILTER {
	public:

		//! Constructor.
		//! If PassOpacity is specified as true and NumTgtSamples is non-zero then NumTgtSamples should
		//! not include any target opacity samples.  In addition, all opacity samples must come after
		//! non-opacity samples in the source and will be placed after non-opacity samples in target.
		//! If PassOpacity is false then source opacity samples are processed and counted the same as all
		//! other sample types when determining default number of target samples.
		FILTER_LINEAR (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			UINT16 NumTgtSamples = 0,				//!< Number of target samples, 0 to determine from previous stage
			bool PassOpacity = true					//!< Pass opacity samples unchanged
			);

		//! Destructor.
		virtual ~FILTER_LINEAR ();

		//! Get coefficient in linear combination matrix.
		//! If either sample index is out of range then 0 will be returned.
		double GetCoefficient (
			UINT16 TgtSampleNum,						//!< Target sample index
			UINT16 SrcSampleNum						//!< Source sample index
			) const;

		//! Get coefficients as MIMATRIX.
		void GetCoefficients (
			MIMATRIX& Coefficients					//!< Coefficient matrix returned
			) const;

		//! Get offset for target sample.
		//! If sample index is out of range then 0 will be returned.
		double GetOffset (
			UINT16 TgtSampleNum						//!< Target sample index
			) const;

		//! Get offsets as MIMATRIX.
		void GetOffsets (
			MIMATRIX& Offsets							//!< Offset matrix returned
			) const;

		//! Set coefficient in linear combination matrix.
		ERRVALUE SetCoefficient (
			UINT16 TgtSampleNum,						//!< Target sample index
			UINT16 SrcSampleNum,						//!< Source sample index
			double value								//!< Value to set
			);

		//! Set coefficients from matrix.
		//! Columns in matrix correspond to source sample indices.
		//! Rows in matrix correspond to target sample indices.
		ERRVALUE SetCoefficients (
			const MIMATRIX& Coefficients			//!< Coefficient matrix
			);

		//! Set offset for target sample.
		ERRVALUE SetOffset (
			UINT16 TgtSampleNum,						//!< Target sample index
			double value								//!< Value to set
			);

		//! Set offsets for target samples.
		//! Matrix must be vector of with size corresponding to number of target samples.
		ERRVALUE SetOffsets (
			const MIMATRIX& Offsets					//!< Offsets as vector matrix
			);

		//! Set offsets for target samples.
		void SetOffsets (
			const double *Offsets					//!< Offsets
			);

	protected:

		// STAGE overrides.

		//! Initialize stage before insertion into pipeline.
		//! Derived class must call base method before doing own processing.
		//! Pointer to source stage should be re-obtained if needed afterward as this
		//! method will insert intermediate filter if required.
		virtual ERRVALUE v_Initialize ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DOUBLE_ARRAY<double> m_Coeff;
		DOUBLE_ARRAY<double> m_Offset;
		UINT16 m_NumSrcSamples;
		UINT16 m_NumTgtSamples;
		UINT16 m_NumOpacity;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_LINEAR (const FILTER_LINEAR&);
		FILTER_LINEAR& operator= (const FILTER_LINEAR&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERLINEAR_H

